#!/usr/bin/env python

# Script for computing precision and recall. It takes as input:
# - a gold standard file in bioboxes format
# (https://github.com/bioboxes/rfc/blob/4bb19a633a6a969c2332f1f298852114c5f89b1b/data-format/binning.mkd)
# with optional column _LENGTH
# - a (compressed) fasta or fastq file, required if _LENGTH is not present in the gold standard file
# - the bins to be evaluated in the same format as above
# It writes to standard output a table containing precision and recall for each bin.

import argparse
import collections
import os, sys, inspect
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0, parentdir)
import numpy as np
import pandas as pd
from src.utils import argparse_parents
from src.utils import load_data


def map_genomes_by_recall(gold_standard, bin_id_to_list_of_sequence_id):
    """
    This script maps a predicted bin to the genome that maximizes recall

    @attention: In case of reads, read ids might not be paired read id and cause error: ReadID/1 ReadID/2

    @param sequence_id_to_genome_id:
    @param anonymous_sequence_id_to_lengths:
    @param bin_id_to_list_of_sequence_id
    @return:
    """
    bin_id_to_genome_id_to_total_length = {}
    mapped_genomes = set()
    bin_id_to_mapped_genome = {}
    for predicted_bin in bin_id_to_list_of_sequence_id:
        bin_id_to_genome_id_to_total_length[predicted_bin] = {}
        for sequence_id in bin_id_to_list_of_sequence_id[predicted_bin]:
            genome_id = gold_standard.sequence_id_to_genome_id[sequence_id]
            if genome_id not in bin_id_to_genome_id_to_total_length[predicted_bin]:
                bin_id_to_genome_id_to_total_length[predicted_bin][genome_id] = 0
            bin_id_to_genome_id_to_total_length[predicted_bin][genome_id] += gold_standard.sequence_id_to_lengths[sequence_id]
        max_genome_percentage = .0
        best_genome_id = ""
        for genome_id in bin_id_to_genome_id_to_total_length[predicted_bin]:
            genome_percentage = bin_id_to_genome_id_to_total_length[predicted_bin][genome_id] / gold_standard.genome_id_to_total_length[genome_id]
            if max_genome_percentage < genome_percentage:
                max_genome_percentage = genome_percentage
                best_genome_id = genome_id
            elif max_genome_percentage == genome_percentage and gold_standard.genome_id_to_total_length[genome_id] > gold_standard.genome_id_to_total_length[best_genome_id]:
                best_genome_id = genome_id
        mapped_genomes.add(best_genome_id)
        bin_id_to_mapped_genome[predicted_bin] = best_genome_id
    return bin_id_to_mapped_genome, bin_id_to_genome_id_to_total_length, mapped_genomes


def map_genomes(gold_standard, bin_id_to_list_of_sequence_id):
    """
    This script maps a predicted bin to the genome that maximizes precision

    @attention: In case of reads, read ids might not be paired read id and cause error: ReadID/1 ReadID/2

    @param sequence_id_to_genome_id:
    @param anonymous_sequence_id_to_lengths:
    @param bin_id_to_list_of_sequence_id
    @return:
    """
    bin_id_to_genome_id_to_total_length = {}
    mapped = set()
    bin_id_to_mapped_genome = {}
    for predicted_bin in bin_id_to_list_of_sequence_id:
        bin_id_to_genome_id_to_total_length[predicted_bin] = {}
        for sequence_id in bin_id_to_list_of_sequence_id[predicted_bin]:
            genome_id = gold_standard.sequence_id_to_genome_id[sequence_id]
            if genome_id not in bin_id_to_genome_id_to_total_length[predicted_bin]:
                bin_id_to_genome_id_to_total_length[predicted_bin][genome_id] = 0
            bin_id_to_genome_id_to_total_length[predicted_bin][genome_id] += gold_standard.sequence_id_to_lengths[sequence_id]
        max_length = 0
        best_genome_id = ""
        for genome_id in bin_id_to_genome_id_to_total_length[predicted_bin]:
            if max_length < bin_id_to_genome_id_to_total_length[predicted_bin][genome_id]:
                max_length = bin_id_to_genome_id_to_total_length[predicted_bin][genome_id]
                best_genome_id = genome_id
        mapped.add(best_genome_id)
        bin_id_to_mapped_genome[predicted_bin] = best_genome_id
    return bin_id_to_mapped_genome, bin_id_to_genome_id_to_total_length, mapped


def compute_confusion_matrix(bin_id_to_mapped_genome,
                             bin_id_to_genome_id_to_total_length,
                             gold_standard,
                             query):
    df_confusion = pd.DataFrame(bin_id_to_genome_id_to_total_length).T

    query_sequence_ids = set(query.sequence_id_to_bin_id.keys())
    gs_sequence_ids = set(gold_standard.sequence_id_to_lengths.keys())
    genome_id_to_unassigned_bps = collections.Counter()
    for unassigned_seq_id in gs_sequence_ids - query_sequence_ids:
        genome_id = gold_standard.sequence_id_to_genome_id[unassigned_seq_id]
        genome_id_to_unassigned_bps[genome_id] += gold_standard.sequence_id_to_lengths[unassigned_seq_id]

    df_unassigned = pd.DataFrame.from_dict(genome_id_to_unassigned_bps, orient='index').rename(columns={0: 'unassigned'}).T
    table = df_confusion.append(df_unassigned)
    table.fillna(0, inplace=True)
    # use log scale
    #table = table.applymap(np.log).fillna(0)

    # sort bins by the number of true positives (length of mapped genome within the bin)
    bin_id_to_mapped_genome_by_length = collections.OrderedDict(sorted(bin_id_to_mapped_genome.items(), key=lambda t: bin_id_to_genome_id_to_total_length[t[0]][t[1]], reverse=True))

    # sort genomes
    genome_order = []
    for bin_id in bin_id_to_mapped_genome_by_length:
        mapped_genome = bin_id_to_mapped_genome_by_length[bin_id]
        if mapped_genome not in genome_order:
            genome_order.append(mapped_genome)
    genome_order += list(set(table.columns.values.tolist()) - set(genome_order))
    for genome_id in genome_id_to_unassigned_bps.keys():
        if genome_id not in genome_order:
            genome_order.append(genome_id)

    table = table.loc[list(bin_id_to_mapped_genome_by_length.keys()) + ['unassigned'], genome_order]

    for genome_id in gold_standard.genome_id_to_total_length.keys():
        if genome_id not in table.columns.values.tolist():
            table[genome_id] = 0

    return table


def compute_precision_recall(gold_standard,
                             bin_id_to_list_of_sequence_id,
                             bin_id_to_mapped_genome,
                             bin_id_to_genome_id_to_total_length,
                             mapped):
    bin_id_to_total_lengths = {}
    for predicted_bin in bin_id_to_list_of_sequence_id:
        for sequence_id in bin_id_to_list_of_sequence_id[predicted_bin]:
            if predicted_bin not in bin_id_to_total_lengths:
                bin_id_to_total_lengths[predicted_bin] = 0
            bin_id_to_total_lengths[predicted_bin] += gold_standard.sequence_id_to_lengths[sequence_id]

    bin_metrics = []
    for predicted_bin in bin_id_to_list_of_sequence_id:
        best_genome_id = bin_id_to_mapped_genome[predicted_bin]
        # length of genome in bin divided by bin size
        precision = float(bin_id_to_genome_id_to_total_length[predicted_bin][best_genome_id]) / float(bin_id_to_total_lengths[predicted_bin])
        recall = float(bin_id_to_genome_id_to_total_length[predicted_bin][best_genome_id]) / float(gold_standard.genome_id_to_total_length[best_genome_id])
        bin_metrics.append({'mapped_genome': best_genome_id,
                            'purity': precision,
                            'completeness': recall,
                            'predicted_size': bin_id_to_total_lengths[predicted_bin],
                            'correctly_predicted': bin_id_to_genome_id_to_total_length[predicted_bin][best_genome_id],
                            'real_size': gold_standard.genome_id_to_total_length[best_genome_id]})
    for genome_id in gold_standard.genome_id_to_list_of_contigs:
        if genome_id not in mapped:
            bin_metrics.append({'mapped_genome': genome_id,
                                'purity': np.nan,
                                'completeness': .0,
                                'predicted_size': 0,
                                'correctly_predicted': 0,
                                'real_size': gold_standard.genome_id_to_total_length[genome_id]})
    # sort bins by completeness
    return sorted(bin_metrics, key=lambda t: t['completeness'], reverse=True)


def compute_metrics(query, gold_standard, bin_id_to_mapped_genome, bin_id_to_genome_id_to_total_length, mapped_genomes):
    bin_metrics = compute_precision_recall(gold_standard,
                                           query.bin_id_to_list_of_sequence_id,
                                           bin_id_to_mapped_genome,
                                           bin_id_to_genome_id_to_total_length,
                                           mapped_genomes)
    return bin_metrics


def print_metrics(bin_metrics, stream=sys.stdout):
    stream.write("genome\tpurity\tcompleteness\tpredicted_size\tcorrectly_predicted\treal_size\n")
    for bin in bin_metrics:
        stream.write("%s\t%s\t%s\t%s\t%s\t%s\n" % (
            bin['mapped_genome'],
            bin['purity'] if not np.isnan(bin['purity']) else 'NA',
            bin['completeness'],
            bin['predicted_size'],
            bin['correctly_predicted'],
            bin['real_size']))


def main():
    parser = argparse.ArgumentParser(description="Compute table of purity and completeness per genome bin",
                                     parents=[argparse_parents.PARSER_GS])
    parser.add_argument('-m', '--map_by_completeness', help=argparse_parents.HELP_MAP_BY_RECALL, action='store_true')
    args = parser.parse_args()
    if not args.gold_standard_file or not args.bin_file:
        parser.print_help()
        parser.exit(1)
    gold_standard = load_data.get_genome_mapping(args.gold_standard_file, args.fasta_file)
    query = load_data.open_query(args.bin_file)

    if args.map_by_completeness:
            bin_id_to_mapped_genome, bin_id_to_genome_id_to_total_length, mapped_genomes = map_genomes_by_recall(gold_standard, query.bin_id_to_list_of_sequence_id)
    else:
        bin_id_to_mapped_genome, bin_id_to_genome_id_to_total_length, mapped_genomes = map_genomes(gold_standard, query.bin_id_to_list_of_sequence_id)

    bin_metrics = compute_metrics(query, gold_standard, bin_id_to_mapped_genome, bin_id_to_genome_id_to_total_length, mapped_genomes)
    print_metrics(bin_metrics)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import argparse
import matplotlib
matplotlib.use('Agg')
import seaborn as sns
import matplotlib.pyplot as plt
from matplotlib.lines import Line2D
import matplotlib.ticker as ticker
import numpy as np
import math
import re
import os, sys, inspect
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0, parentdir)
from src.utils import load_data
from src.utils import labels

LEGEND2 = False


def create_colors_list():
    colors_list = []
    for color in plt.cm.tab10(np.linspace(0, 1, 10))[:-1]:
        colors_list.append(tuple(color))
    colors_list.append("black")
    for color in plt.cm.Set2(np.linspace(0, 1, 8)):
        colors_list.append(tuple(color))
    for color in plt.cm.Set3(np.linspace(0, 1, 12)):
        colors_list.append(tuple(color))
    return colors_list


def load_results(files):
    table = []
    for file in files:
        f = open(file, 'r')
        field_names = f.readline().rstrip('\n').split('\t')
        field_names[field_names.index('tool')] = 'binning_label'
        for line in f:
            values = line.rstrip('\n').split('\t')
            results_dict = dict(zip(field_names, values))
            table.append(results_dict)
        f.close()
    return table


def scan_dir(output_dir):
    p_order = re.compile('^#\([0-9]+\)')
    order = []
    data_list = []
    binning_labels = []
    for path in [d for d in (os.path.join(output_dir, d1) for d1 in os.listdir(output_dir)) if os.path.isdir(d)]:
        f = open(os.path.join(path, 'purity_completeness.tsv'), 'r')
        data_list.append(load_data.load_tsv_table(f))

        # load label and order
        f = open(os.path.join(path, 'label.txt'), 'r')
        line = f.readline().rstrip('\n')
        match = p_order.match(line)
        match_string = match.group()
        order.append(int(match_string[2:match.end() - 1]))
        binning_labels.append(line[match.end():])
        f.close()

    return data_list, binning_labels, order


def plot_heatmap(df_confusion, output_dir, separate_bar=False):
    fig, axs = plt.subplots(figsize=(10, 8))

    sns_plot = sns.heatmap(df_confusion, ax=axs, annot=False, cmap="YlGnBu_r", xticklabels=False, yticklabels=False, cbar=True)
    sns_plot.set_xlabel("Genomes", fontsize=20)
    sns_plot.set_ylabel("Predicted bins", fontsize=20)
    # plt.yticks(fontsize=8, rotation=0)
    # plt.xticks(fontsize=8)

    fig.savefig(os.path.normpath(output_dir + '/heatmap.eps'), dpi=100, format='eps', bbox_inches='tight')
    fig.savefig(os.path.normpath(output_dir + '/heatmap.png'), dpi=100, format='png', bbox_inches='tight')
    plt.close(fig)

    if not separate_bar:
        return

    # create separate figure for bar
    fig = plt.figure(figsize=(6, 6))
    mappable = sns_plot.get_children()[0]
    fmt = lambda x, pos: '{:.0f}'.format(x / 1000000)

    cbar = plt.colorbar(mappable, orientation='vertical', label='[millions]', format=ticker.FuncFormatter(fmt))

    text = cbar.ax.yaxis.label
    font = matplotlib.font_manager.FontProperties(size=16)
    text.set_font_properties(font)

    cbar.outline.set_visible(False)
    cbar.ax.tick_params(labelsize=14)

    # store separate bar figure
    plt.gca().set_visible(False)
    fig.savefig(os.path.normpath(output_dir + '/heatmap_bar.eps'), dpi=100, format='eps', bbox_inches='tight')

    plt.close(fig)


def plot_boxplot(data_list, binning_labels, metric_name, output_dir, order=None):
    precision_all = []
    for metrics in data_list:
        precision = []
        for metric in metrics:
            if not math.isnan(metric[metric_name]):
                precision.append(metric[metric_name])
        precision_all.append(precision)

    if order:
        # sort binning_labels and precision_all by order
        enum_order = [(v, k) for k, v in enumerate(order)]
        enum_order = sorted(enum_order, key=lambda x: x[0])
        binning_labels = [binning_labels[i[1]] for i in enum_order]
        precision_all = [precision_all[i[1]] for i in enum_order]

    fig, axs = plt.subplots(figsize=(6, 5))

    medianprops = dict(linewidth=2.5, color='gold')
    bplot = axs.boxplot(precision_all, notch=0, vert=0, patch_artist=True, labels=binning_labels, medianprops=medianprops, sym='k.')
    colors_iter = iter(create_colors_list())

    # turn on grid
    axs.grid(which='major', linestyle='-', linewidth='0.5', color='lightgrey')

    # force axes to be from 0 to 100%
    axs.set_xlim([-0.01, 1.01])

    # transform plot_labels to percentages
    vals = axs.get_xticks()
    axs.set_xticklabels(['{:3.0f}'.format(x * 100) for x in vals])

    # enable code to rotate labels
    tick_labels = axs.get_yticklabels()
    plt.setp(tick_labels, fontsize=14) ## rotation=55

    for box in bplot['boxes']:
        box.set(facecolor=next(colors_iter), linewidth=0.1)
    plt.ylim(plt.ylim()[::-1])

    if metric_name == 'purity':
        axs.set_xlabel('Purity per bin $p$ (%)' if LEGEND2 else 'Purity per bin (%)', fontsize=14)
    else:
        axs.set_xlabel('Completeness per genome $r$ (%)' if LEGEND2 else 'Completeness per genome (%)', fontsize=14)

    fig.savefig(os.path.normpath(output_dir + '/boxplot_' + metric_name + '.pdf'), dpi=100, format='pdf', bbox_inches='tight')
    fig.savefig(os.path.normpath(output_dir + '/boxplot_' + metric_name + '.png'), dpi=100, format='png', bbox_inches='tight')
    fig.savefig(os.path.normpath(output_dir + '/boxplot_' + metric_name + '.eps'), dpi=100, format='eps', bbox_inches='tight')

    # remove labels but keep grid
    axs.get_yaxis().set_ticklabels([])
    for tic in axs.yaxis.get_major_ticks():
        tic.tick1On = tic.tick2On = False
        tic.label1On = tic.label2On = False
    fig.savefig(os.path.normpath(output_dir + '/boxplot_' + metric_name + '_wo_legend.eps'), dpi=100, format='eps', bbox_inches='tight')
    plt.close(fig)


def plot_summary(summary_per_query, output_dir, plot_type, file_name, xlabel, ylabel):
    colors_list = create_colors_list()
    if len(summary_per_query) > len(colors_list):
        raise RuntimeError("Plot only supports 29 colors")

    fig, axs = plt.subplots(figsize=(6, 5))

    # force axes to be from 0 to 100%
    axs.set_xlim([0.0, 1.0])
    axs.set_ylim([0.0, 1.0])

    i = 0
    plot_labels = []
    if plot_type == 'e':
        for summary in summary_per_query:
            axs.errorbar(float(summary[labels.AVG_PRECISION]), float(summary[labels.AVG_RECALL]), xerr=float(summary[labels.SEM_PRECISION]), yerr=float(summary[labels.SEM_RECALL]),
                         fmt='o',
                         ecolor=colors_list[i],
                         mec=colors_list[i],
                         mfc=colors_list[i],
                         capsize=3,
                         markersize=8)
            plot_labels.append(summary[labels.TOOL])
            i += 1
    if plot_type == 'w':
        for summary in summary_per_query:
            axs.plot(float(summary[labels.PRECISION]), float(summary[labels.RECALL]), marker='o', color=colors_list[i], markersize=10)
            plot_labels.append(summary[labels.TOOL])
            i += 1
    elif plot_type == 'p':
        for summary in summary_per_query:
            axs.plot(float(summary[labels.ARI_BY_BP]), float(summary[labels.PERCENTAGE_ASSIGNED_BPS]), marker='o', color=colors_list[i], markersize=10)
            plot_labels.append(summary[labels.TOOL])
            i += 1

    # turn on grid
    axs.minorticks_on()
    axs.grid(which='major', linestyle='-', linewidth='0.5')
    axs.grid(which='minor', linestyle=':', linewidth='0.5')

    # transform plot_labels to percentages
    vals = axs.get_xticks()
    axs.set_xticklabels(['{:3.0f}'.format(x * 100) for x in vals])
    vals = axs.get_yticks()
    axs.set_yticklabels(['{:3.0f}'.format(x * 100) for x in vals])

    plt.xlabel(xlabel, fontsize=14)
    plt.ylabel(ylabel, fontsize=14)
    plt.tight_layout()
    fig.savefig(os.path.normpath(output_dir + '/' + file_name + '.eps'), dpi=100, format='eps', bbox_inches='tight')

    colors_iter = iter(colors_list)
    circles = []
    for summary in summary_per_query:
        circles.append(Line2D([], [], markeredgewidth=0.0, linestyle="None", marker="o", markersize=11, markerfacecolor=next(colors_iter)))

    lgd = plt.legend(circles, plot_labels, bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0., handlelength=0, frameon=False, fontsize=12)

    fig.savefig(os.path.normpath(output_dir + '/' + file_name + '.png'), dpi=100, format='png', bbox_extra_artists=(lgd,), bbox_inches='tight')
    fig.savefig(os.path.normpath(output_dir + '/' + file_name + '.pdf'), dpi=100, format='pdf', bbox_extra_artists=(lgd,), bbox_inches='tight')
    plt.close(fig)


def plot_avg_precision_recall(summary_per_query, output_dir):
    plot_summary(summary_per_query,
                 output_dir,
                 'e',
                 'avg_purity_completeness',
                 'Truncated average purity per bin $\overline{p}_{99}$ (%)' if LEGEND2 else 'Average purity per bin (%)',
                 'Average completeness per genome $\overline{r}$ (%)' if LEGEND2 else 'Average completeness per genome (%)')


def plot_weighed_precision_recall(summary_per_query, output_dir):
    plot_summary(summary_per_query,
                 output_dir,
                 'w',
                 'avg_purity_completeness_per_bp',
                 'Average purity per base pair $\overline{p}_{bp}$ (%)' if LEGEND2 else 'Average purity per base pair (%)',
                 'Average completeness per base pair $\overline{r}_{bp}$ (%)' if LEGEND2 else 'Average completeness per base pair (%)')


def plot_adjusted_rand_index_vs_assigned_bps(summary_per_query, output_dir):
    plot_summary(summary_per_query,
                 output_dir,
                 'p',
                 'ari_vs_assigned_bps',
                 'Adjusted Rand Index (%)' if LEGEND2 else 'Adjusted Rand Index',
                 'Percentage of assigned base pairs (%)' if LEGEND2 else 'Percentage of assigned base pairs')


def main():
    parser = argparse.ArgumentParser(description="Create plots from one or more tables of results")
    parser.add_argument("files", nargs='+', help="File(s) including system path")
    parser.add_argument('-o', '--output_dir', help="Directory to save the plots in", required=True)
    args = parser.parse_args()
    load_data.make_sure_path_exists(args.output_dir)
    results = load_results(args.files)
    plot_avg_precision_recall(results, args.output_dir)
    plot_weighed_precision_recall(results, args.output_dir)
    plot_adjusted_rand_index_vs_assigned_bps(results, args.output_dir)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import argparse

HELP_GOLD_STANDARD_FILE = "Gold standard - ground truth - file"
HELP_FILE = "File containing precision and recall for each genome"
HELP_QUERY_FILE = "Binning file"
HELP_QUERY_FILES = "Binning files"
HELP_FASTA_FILE = "FASTA or FASTQ file with sequences of gold standard (required if gold standard file misses column _LENGTH)"
HELP_FILTER = "Filter out [FILTER]%% smallest bins (default: 0)"
HELP_GENOMES_FILE = "File with list of genomes to be removed"
HELP_LABEL = "Binning name"
HELP_LABELS = "Comma-separated binning names"
HELP_KEYWORD = "Keyword in the second column of file with list of genomes to be removed (no keyword=remove all genomes in list)"
HELP_MAP_BY_RECALL = "Map genomes to bins by maximizing completeness"
HELP_THRESHOLDS_COMPLETENESS = "Comma-separated list of min. completeness thresholds (default %%: 50,70,90)"
HELP_THRESHOLDS_CONTAMINATION = "Comma-separated list of max. contamination thresholds (default %%: 10,5)"

PARSER_GS = argparse.ArgumentParser(add_help=False)
PARSER_GS.add_argument("bin_file", help=HELP_QUERY_FILE)
PARSER_GS.add_argument("-g", "--gold_standard_file", help=HELP_GOLD_STANDARD_FILE, required=True)
PARSER_GS.add_argument("-f", "--fasta_file", help=HELP_FASTA_FILE)

PARSER_MULTI = argparse.ArgumentParser(add_help=False)
PARSER_MULTI.add_argument('file', nargs='?', type=argparse.FileType('r'), help=HELP_FILE)
PARSER_MULTI.add_argument('-p', '--filter', help=HELP_FILTER)
PARSER_MULTI.add_argument('-l', '--label', help=HELP_LABEL, required=False)

PARSER_MULTI2 = argparse.ArgumentParser(add_help=False)
PARSER_MULTI2.add_argument("bin_files", nargs='+', help=HELP_QUERY_FILES)
PARSER_MULTI2.add_argument("-g", "--gold_standard_file", help=HELP_GOLD_STANDARD_FILE, required=True)
PARSER_MULTI2.add_argument("-f", "--fasta_file", help=HELP_FASTA_FILE)
PARSER_MULTI2.add_argument('-l', '--labels', help=HELP_LABELS, required=False)
PARSER_MULTI2.add_argument('-p', '--filter', help=HELP_FILTER)
PARSER_MULTI2.add_argument('-r', '--remove_genomes', help=HELP_GENOMES_FILE, required=False)
PARSER_MULTI2.add_argument('-k', '--keyword', help=HELP_KEYWORD, required=False)



#!/usr/bin/env python

import argparse
import sys
import os

try:
    import argparse_parents
    import load_data
except ImportError:
    sys.path.append(os.path.dirname(__file__))
    try:
        import argparse_parents
        import load_data
    finally:
        sys.path.remove(os.path.dirname(__file__))


def add_column(mapping_file, fasta_file):
    lengths = load_data.read_lengths_from_fastx_file(fasta_file)
    sequence_id_column_index = 0
    with open(mapping_file, 'r') as read_handler:
        for line in read_handler:
            line = line.rstrip('\n')
            if line.startswith('@@'):
                for index, column_name in enumerate(line[2:].split('\t')):
                    if column_name == 'SEQUENCEID':
                        sequence_id_column_index = index
                print('%s\t_LENGTH' % line)
                continue

            if len(line.strip()) == 0 or line.startswith('#') or line.startswith('@'):
                print(line)
                continue

            row_data = line.split('\t')
            sequence_id = row_data[sequence_id_column_index]
            print('%s\t%d' % (line, lengths[sequence_id]))


def main():
    parser = argparse.ArgumentParser(description="Add length column _LENGTH to gold standard mapping and print mapping on the standard output")
    parser.add_argument("-g", "--gold_standard_file", help=argparse_parents.HELP_GOLD_STANDARD_FILE, required=True)
    parser.add_argument("-f", "--fasta_file", help="FASTA or FASTQ file with sequences of gold standard", required=True)
    args = parser.parse_args()
    add_column(args.gold_standard_file, args.fasta_file)


if __name__ == "__main__":
    main()


#!/usr/bin/env python

import collections

TOOL = "tool"
PRECISION = "avg_purity_per_bp"
RECALL = "avg_completeness_per_bp"
AVG_PRECISION = "avg_purity"
STD_DEV_PRECISION = "std_dev_purity"
SEM_PRECISION = "sem_purity"
AVG_RECALL = "avg_completeness"
STD_DEV_RECALL = "std_dev_completeness"
SEM_RECALL = "sem_completeness"
RI_BY_BP = "rand_index_by_bp"
RI_BY_SEQ = "rand_index_by_seq"
ARI_BY_BP = "a_rand_index_by_bp"
ARI_BY_SEQ = "a_rand_index_by_seq"
PERCENTAGE_ASSIGNED_BPS = "percent_assigned_bps"
ACCURACY = "accuracy"

abbreviations = collections.OrderedDict([(AVG_PRECISION, 'purity averaged over genome bins'),
                 (STD_DEV_PRECISION, 'standard deviation of purity averaged over genome bins'),
                 (SEM_PRECISION, 'standard error of the mean of purity averaged over genome bins'),
                 (AVG_RECALL, 'completeness averaged over genome bins'),
                 (STD_DEV_RECALL, 'standard deviation of completeness averaged over genome bins'),
                 (SEM_RECALL, 'standard error of the mean of completeness averaged over genome bins'),
                 (PRECISION, 'average purity per base pair'),
                 (RECALL, 'average completeness per base pair'),
                 ('rand_index_by_bp', 'Rand index weighed by base pairs'),
                 ('rand_index_by_seq', 'Rand index weighed by sequence counts'),
                 ('a_rand_index_by_bp', 'adjusted Rand index weighed by base pairs'),
                 ('a_rand_index_by_seq', 'adjusted Rand index weighed by sequence counts'),
                 ('percent_assigned_bps', 'percentage of base pairs that were assigned to bins'),
                 ('accuracy', 'accuracy'),
                 ('>0.5compl<0.1cont', 'number of bins with more than 50% completeness and less than 10% contamination'),
                 ('>0.7compl<0.1cont', 'number of bins with more than 70% completeness and less than 10% contamination'),
                 ('>0.9compl<0.1cont', 'number of bins with more than 90% completeness and less than 10% contamination'),
                 ('>0.5compl<0.05cont', 'number of bins with more than 50% completeness and less than 5% contamination'),
                 ('>0.7compl<0.05cont', 'number of bins with more than 70% completeness and less than 5% contamination'),
                 ('>0.9compl<0.05cont', 'number of bins with more than 90% completeness and less than 5% contamination')])



#!/usr/bin/env python

import argparse


def read_fasta_file(fasta_file):
    with open(fasta_file, 'r') as read_handler:
        sequence_id = ""
        for line in read_handler:
            line = line.strip()
            if not line:
                continue
            if line.startswith(">"):
                if len(sequence_id) > 0:
                    yield sequence_id, fasta_file
                sequence_id = line[1:]
                continue
        if len(sequence_id) > 0:
            yield sequence_id, fasta_file


def convert(paths, output_file):
    with open(output_file, 'w') as write_handler:
        write_handler.write("#CAMI Format for Binning\n@Version:0.9.0\n@SampleID:_SAMPLEID_\n@@SEQUENCEID\tBINID\n")
        for file in paths:
            for sequence_id, bin_id in read_fasta_file(file):
                write_handler.write("%s\t%s\n" % (sequence_id, bin_id))


def main():
    parser = argparse.ArgumentParser(description="Convert bins in FASTA files to CAMI tsv format")
    parser.add_argument("paths", nargs='+', help="FASTA files including full paths")
    parser.add_argument("-o", "--output_file", required=False, help="Output file")
    args = parser.parse_args()
    convert(args.paths, args.output_file)


if __name__ == "__main__":
    main()


#!/usr/bin/env python

import argparse
import os, sys, inspect
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0, parentdir)
from src.utils import argparse_parents
from src.utils import load_data

PLOTS = ['avg_purity_completeness',
         'ari_vs_assigned_bps',
         'avg_purity_completeness_per_bp',
         'completeness_contamination_table',
         'purity_completeness_per_bin']


def load_summary(input_dir):
    f = open(input_dir + "/summary.tsv", 'r')
    field_names = f.readline().rstrip('\n').split('\t')
    table = []
    for line in f:
        values = line.rstrip('\n').split('\t')
        table.append(dict(zip(field_names, values)))
    f.close()
    return table


def create_completeness_contamination_table(input_dir, output_dir, gold_standard=None):
    summary = load_summary(input_dir)
    with open(output_dir + "/completeness_contamination_table.tex", "w") as texfile:
        texfile.write('{}{}'.format(r'\documentclass{article}', '\n'))
        texfile.write('{}{}'.format(r'\usepackage[T1]{fontenc}', '\n'))
        texfile.write('{}{}'.format(r'\usepackage{multirow}', '\n'))
        texfile.write('{}{}'.format(r'\pagestyle{empty}', '\n'))
        texfile.write('{}{}'.format(r'\begin{document}', '\n'))
        texfile.write('{}{}'.format(r'\fontfamily{cmss}\selectfont', '\n'))
        texfile.write('{}{}'.format(r'\begin{tabular}{l|r|r|r|r}', '\n'))
        texfile.write('{}{}'.format(r'\multicolumn{2}{c|}{\textbf{Genome binner}}&\multicolumn{3}{|c}{\textbf{Predicted bins}}\\', '\n'))
        texfile.write('{}{}'.format(r'\multicolumn{2}{c|}{\textbf{(\% contamination)}}&\multicolumn{3}{|c}{\textbf{(\% completeness)}}\\', '\n'))
        texfile.write('{}{}'.format(r'\cline{3-5}', '\n'))
        texfile.write('{}{}'.format(r'\multicolumn{2}{c|}{}&>50\%&>70\%&>90\%\\', '\n'))
        texfile.write('{}{}'.format(r'\hline', '\n'))

        if gold_standard:
            num_genomes = str(len(gold_standard.genome_id_to_list_of_contigs))
            texfile.write('{}{}'.format(r'\multicolumn{2}{l|}{}&&&\\[-1em]', '\n'))
            texfile.write('{}{}'.format(r'\multicolumn{2}{l|}{Gold standard}&' + num_genomes + r'&' + num_genomes + r'&' + num_genomes + r'\\', '\n'))
            texfile.write('{}{}'.format(r'\hline', '\n'))

        for row in summary:
            texfile.write('{}{}'.format(r'\multirow{2}{*}{' + row['tool'] + r'}&<10\%&' + row['>0.5compl<0.1cont'] + r'&' + row['>0.7compl<0.1cont'] + r'&' + row['>0.9compl<0.1cont'] + r'\\', '\n'))
            texfile.write('{}{}'.format(r'&<5\%&' + row['>0.5compl<0.05cont'] + r'&' + row['>0.7compl<0.05cont'] + r'&' + row['>0.9compl<0.05cont'] + r'\\', '\n'))
            texfile.write('{}{}'.format(r'\hline', '\n'))
        texfile.write('{}{}'.format(r'\end{tabular}', '\n'))
        texfile.write('{}{}'.format(r'\end{document}', '\n'))
    texfile.close()
    os.system("latex -output-directory " + output_dir + " " + output_dir + "/completeness_contamination_table.tex")
    os.system("dvips " + output_dir + "/completeness_contamination_table.dvi " + "-o " + output_dir + "/completeness_contamination_table.ps")
    os.system("ps2eps -f " + output_dir + "/completeness_contamination_table.ps")
    os.system("cp " + output_dir + "/completeness_contamination_table.eps " + input_dir)


def create_pdf(input_dir, output_dir, plots):
    with open(output_dir + "/summary.tex", "w") as texfile:
        texfile.write('{}{}'.format(r'\documentclass{article}', '\n'))
        texfile.write('{}{}'.format(r'\usepackage[left=0.3in,right=0.3in,top=0.5in,bottom=0.3in]{geometry}', '\n'))
        texfile.write('{}{}'.format(r'\usepackage{graphicx}', '\n'))
        texfile.write('{}{}'.format(r'\usepackage{floatrow}', '\n'))
        texfile.write('{}{}'.format(r'\usepackage{subfig}', '\n'))
        texfile.write('{}{}'.format(r'\usepackage[T1]{fontenc}', '\n'))
        texfile.write('{}{}'.format(r'\usepackage{multirow}', '\n'))
        texfile.write('{}{}'.format(r'\floatsetup[figure]{style=plain,subcapbesideposition=top}', '\n'))
        texfile.write('{}{}'.format(r'\begin{document}', '\n'))
        texfile.write('{}{}'.format(r'\begin{figure}[!ht]', '\n'))
        texfile.write('{}{}'.format(r'\centering', '\n'))
        texfile.write('{}{}'.format(r'\includegraphics[scale=0.75]{' + input_dir + '/legend}\\', '\n'))
        texfile.write('{}{}'.format(r'\ffigbox', '\n'))
        texfile.write('{}{}'.format(r'{', '\n'))
        i = 1
        for plot in plots:
            if i % 2 != 0:
                texfile.write('{}{}'.format(r'\begin{subfloatrow}', '\n'))
            texfile.write('{}{}'.format(r'\sidesubfloat[]{\includegraphics[width=0.45\linewidth]{' + input_dir + '/' + PLOTS[plot - 1] + r'}}', '\n'))
            if i % 2 == 0 or i == len(plots):
                texfile.write('{}{}'.format(r'\end{subfloatrow}', '\n'))
                if i < len(plots):
                    texfile.write('{}{}'.format(r'\par\bigskip', '\n'))
            i += 1
        texfile.write('{}{}'.format(r'}', '\n'))
        texfile.write('{}{}'.format(r'{}', '\n'))
        texfile.write('{}{}'.format(r'\end{figure}', '\n'))
        texfile.write('{}{}'.format(r'\end{document}', '\n'))
    texfile.close()
    os.system("latex -output-directory " + output_dir + " " + output_dir + "/summary.tex")
    os.system("dvipdf " + output_dir  + "/summary.dvi " + output_dir + "/summary.pdf")


def main():
    if os.name == 'nt':
        exit('Sorry, this tool does not support Windows.')
    parser = argparse.ArgumentParser(description="Combine figures and table of completeness and contamination into file summary.pdf")
    parser.add_argument('-c', '--figure_codes', help="Comma-separated figure codes without spaces (1=average purity vs. average completeness, 2=ari vs. %%assigned bps, 3=weighed purity vs. weighed completeness, 4=table of contamination and completeness, 5=purity vs. completeness per bin; default 1,2,3,4)", required=False)
    parser.add_argument('-i', '--input_dir', help="Directory to read file summary.tsv and figures from", required=True)
    parser.add_argument('-o', '--output_dir', help="Directory to write pdf and temporary files (if different from input_dir)", required=False)
    parser.add_argument('-g', '--gold_standard_file', help=argparse_parents.HELP_GOLD_STANDARD_FILE, required=False)
    parser.add_argument('-r', '--remove_genomes', help=argparse_parents.HELP_GENOMES_FILE, required=False)
    parser.add_argument('-k', '--keyword', help=argparse_parents.HELP_KEYWORD, required=False)

    args = parser.parse_args()
    figure_codes = [1, 2, 3, 4]
    output_dir = os.path.abspath(args.input_dir)
    input_dir = os.path.abspath(args.input_dir)

    if args.figure_codes:
        figure_codes = [int(x.strip()) for x in args.figure_codes.split(',')]
    if args.output_dir:
        output_dir = os.path.abspath(args.output_dir)
        load_data.make_sure_path_exists(args.output_dir)

    gold_standard = None
    if args.gold_standard_file:
        gold_standard = load_data.get_genome_mapping_without_lenghts(args.gold_standard_file, args.remove_genomes, args.keyword)

    if 4 in figure_codes:
        create_completeness_contamination_table(input_dir, output_dir, gold_standard)

    create_pdf(input_dir, output_dir, figure_codes)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import sys
import os
import argparse

try:
    import argparse_parents
    import load_data
except ImportError:
    sys.path.append(os.path.dirname(__file__))
    try:
        import argparse_parents
        import load_data
    finally:
        sys.path.remove(os.path.dirname(__file__))


def print_filtered_data(stream, unique_common_file_path, keyword):
    genome_to_unique_common = load_data.load_unique_common(unique_common_file_path)
    for line in stream:
        line = line.strip()
        if len(line) == 0 or line.startswith("@"):
            print(line)
            continue
        bin = line.split('\t')[0]
        if bin in genome_to_unique_common and (keyword is None or genome_to_unique_common[bin] == keyword):
            continue
        print(line)


def filter_genomes(gold_standard, unique_common_file_path, keyword):
    if not unique_common_file_path:
        return gold_standard.genome_id_to_list_of_contigs.keys()
    genome_to_unique_common = load_data.load_unique_common(unique_common_file_path)
    genome_ids = []
    for genome_id in gold_standard.genome_id_to_list_of_contigs:
        if genome_id not in genome_to_unique_common or (keyword and genome_to_unique_common[genome_id] != keyword):
            genome_ids.append(genome_id)
    return genome_ids


def filter_data(bin_metrics, unique_common_file_path, keyword):
    genome_to_unique_common = load_data.load_unique_common(unique_common_file_path)
    filtered_bin_metrics = []
    for bin in bin_metrics:
        bin_id = bin['mapped_genome']
        if bin_id not in genome_to_unique_common or (keyword and genome_to_unique_common[bin_id] != keyword):
            filtered_bin_metrics.append(bin)
    if len(filtered_bin_metrics) == 0:
        sys.exit('All bins have been filtered out due to option --remove_genomes.')
    return filtered_bin_metrics


def main():
    parser = argparse.ArgumentParser(description="Exclude genome bins from table of precision and recall per genome. The table can be provided as file or via the standard input")
    parser.add_argument('file', nargs='?', type=argparse.FileType('r'), help=argparse_parents.HELP_FILE)
    parser.add_argument('-r', '--remove_genomes', help=argparse_parents.HELP_GENOMES_FILE, required=True)
    parser.add_argument('-k', '--keyword', help=argparse_parents.HELP_KEYWORD, required=False)
    args = parser.parse_args()
    if not args.file and sys.stdin.isatty():
        parser.print_help()
        parser.exit(1)
    print_filtered_data(sys.stdin if not sys.stdin.isatty() else args.file,
                        args.remove_genomes,
                        args.keyword)


if __name__ == "__main__":
    main()


#!/usr/bin/env python

import os
import gzip
import mimetypes
import sys
from Bio import SeqIO
import numpy as np
import errno

try:
    import exclude_genomes
except ImportError:
    sys.path.append(os.path.dirname(__file__))
    try:
        import exclude_genomes
    finally:
        sys.path.remove(os.path.dirname(__file__))


def make_sure_path_exists(path):
    try:
        os.makedirs(path)
    except OSError as exception:
        if exception.errno != errno.EEXIST:
            raise


class GoldStandard:
    def __init__(self):
        pass


class Query:
    def __init__(self):
        pass


def load_unique_common(unique_common_file_path):
    genome_to_unique_common = {}
    with open(unique_common_file_path) as read_handler:
        for line in read_handler:
            genome_to_unique_common[line.split("\t")[0]] = line.split("\t")[1].strip('\n')
    return genome_to_unique_common


def load_tsv_table(stream):
    data = []
    next(stream)
    for line in stream:
        line = line.strip()
        if len(line) == 0 or line.startswith("@"):
            continue
        row_data = line.split('\t')

        mapped_genome = row_data[0]
        real_size = int(float(row_data[5]))
        predicted_size = int(float(row_data[3]))
        correctly_predicted = int(float(row_data[4]))

        if row_data[1] != "NA" and predicted_size > 0:
            precision = float(row_data[1])
        else:
            precision = np.nan
        data.append({'mapped_genome': mapped_genome, 'precision': precision, 'recall': float(row_data[2]),
                     'predicted_size': predicted_size, 'correctly_predicted': correctly_predicted, 'real_size': real_size})
    return data


def read_lengths_from_fastx_file(fastx_file):
    """

    @param fastx_file: file path
    @type fastx_file: str
    @rtype: dict[str, int]
    """
    file_type = mimetypes.guess_type(fastx_file)[1]
    if file_type == 'gzip':
        f = gzip.open(fastx_file, "rt")
    elif not file_type:
        f = open(fastx_file, "rt")
    else:
        raise RuntimeError("Unknown type of file: '{}".format(fastx_file))

    length = {}
    if os.path.getsize(fastx_file) == 0:
        return length

    file_format = None
    line = f.readline()
    if line.startswith('@'):
        file_format = "fastq"
    elif line.startswith(">"):
        file_format = "fasta"
    f.seek(0)

    if not file_format:
        raise RuntimeError("Invalid sequence file: '{}".format(fastx_file))

    for seq_record in SeqIO.parse(f, file_format):
        length[seq_record.id] = len(seq_record.seq)

    f.close()

    return length


def get_genome_mapping_without_lenghts(mapping_file, remove_genomes_file=None, keyword=None):
    gold_standard = GoldStandard()
    gold_standard.genome_id_to_list_of_contigs = {}
    gold_standard.sequence_id_to_genome_id = {}

    filtering_genomes_to_keyword = {}
    if remove_genomes_file:
        filtering_genomes_to_keyword = load_unique_common(remove_genomes_file)

    with open(mapping_file, 'r') as read_handler:
        try:
            for sequence_id, genome_id, length in read_binning_file(read_handler):
                if genome_id in filtering_genomes_to_keyword and (not keyword or filtering_genomes_to_keyword[genome_id] == keyword):
                    continue
                gold_standard.sequence_id_to_genome_id[sequence_id] = genome_id
                if genome_id not in gold_standard.genome_id_to_list_of_contigs:
                    gold_standard.genome_id_to_list_of_contigs[genome_id] = []
                gold_standard.genome_id_to_list_of_contigs[genome_id].append(sequence_id)
        except:
            exit("Error. File {} is malformed.".format(mapping_file))

    if len(gold_standard.genome_id_to_list_of_contigs) == 0:
        exit('All bins of the gold standard have been filtered out due to option --remove_genomes.')

    return gold_standard


def get_genome_mapping(mapping_file, fastx_file):
    """
    @param mapping_file:
    @type mapping_file: str | unicode

    @return:
    """
    gold_standard = GoldStandard()
    gold_standard.genome_id_to_total_length = {}
    gold_standard.genome_id_to_list_of_contigs = {}
    gold_standard.sequence_id_to_genome_id = {}
    gold_standard.sequence_id_to_lengths = {}

    with open(mapping_file, 'r') as read_handler:
        is_length_column_av = is_length_column_available(read_handler)
        sequence_length = {}
        if not is_length_column_av:
            if not fastx_file:
                exit("Sequences length could not be determined. Please provide a FASTA or FASTQ file using option -f or add column _LENGTH to gold standard.")
            sequence_length = read_lengths_from_fastx_file(fastx_file)
        try:
            for anonymous_contig_id, genome_id, length in read_binning_file(read_handler, True):
                total_length = length if is_length_column_av else sequence_length[anonymous_contig_id]
                gold_standard.sequence_id_to_lengths[anonymous_contig_id] = total_length
                gold_standard.sequence_id_to_genome_id[anonymous_contig_id] = genome_id
                if genome_id not in gold_standard.genome_id_to_total_length:
                    gold_standard.genome_id_to_total_length[genome_id] = 0
                    gold_standard.genome_id_to_list_of_contigs[genome_id] = []
                gold_standard.genome_id_to_total_length[genome_id] += total_length
                gold_standard.genome_id_to_list_of_contigs[genome_id].append(anonymous_contig_id)
        except KeyError:
            exit("Error. Sequence {} could not be found in the FASTA or FASTQ file.".format(anonymous_contig_id))
        except:
            exit("Error. File {} is malformed.".format(mapping_file))

    return gold_standard


def read_header(input_stream):
    """
    @Version:0.9.1
    @SampleID:RH_S1

    @@SEQUENCEID    BINID   TAXID
    """
    header = {}
    column_names = {}
    for line in input_stream:
        if len(line.strip()) == 0 or line.startswith("#"):
            continue
        line = line.rstrip('\n')
        if line.startswith("@@"):
            for index, column_name in enumerate(line[2:].split('\t')):
                column_names[column_name] = index
            return header, column_names
        if line.startswith("@"):
            key, value = line[1:].split(':', 1)
            header[key] = value.strip()


def read_rows(input_stream, index_key, index_value, index_length, is_gs):
    for line in input_stream:
        if len(line.strip()) == 0 or line.startswith("#"):
            continue
        line = line.rstrip('\n')
        row_data = line.split('\t')

        try:
            key = row_data[index_key]
        except:
            print("Value in column SEQUENCEID could not be read.")
            raise

        try:
            value = row_data[index_value]
        except:
            print("Value in column BINID could not be read.")
            raise

        if is_gs and index_length is not None:
            try:
                length = int(row_data[index_length])
            except:
                print("Value in column _LENGTH could not be read. Please provide a value or remove column altogether (and provide a FASTA or FASTQ file instead - see README).")
                raise
            yield key, value, length
        else:
            yield key, value, int(0)


def is_length_column_available(input_stream):
    header, column_names = read_header(input_stream)
    input_stream.seek(0)
    return "_LENGTH" in column_names


def get_column_indices(input_stream):
    """

    :param input_stream: 
    :return: 
    """
    header, column_names = read_header(input_stream)
    if "SEQUENCEID" not in column_names:
        raise RuntimeError("Column not found: {}".format("SEQUENCEID"))
    index_key = column_names["SEQUENCEID"]
    if "BINID" in column_names:
        index_value = column_names["BINID"]
    elif "TAXID" in column_names:
        index_value = column_names["TAXID"]
    else:
        raise RuntimeError("Column not found: {}".format("BINID/TAXID"))
    index_length = None
    if "_LENGTH" in column_names:
        index_length = column_names["_LENGTH"]
    return index_key, index_value, index_length


def read_binning_file(input_stream, is_gs=False):
    index_key, index_value, index_length = get_column_indices(input_stream)
    return read_rows(input_stream, index_key, index_value, index_length, is_gs)


def open_query(file_path_query):
    query = Query()
    query.path = file_path_query
    query.bin_id_to_list_of_sequence_id = {}
    query.sequence_id_to_bin_id = {}
    with open(file_path_query) as read_handler:
        try:
            for sequence_id, predicted_bin, length in read_binning_file(read_handler):
                if predicted_bin not in query.bin_id_to_list_of_sequence_id:
                    query.bin_id_to_list_of_sequence_id[predicted_bin] = []
                query.bin_id_to_list_of_sequence_id[predicted_bin].append(sequence_id)
                query.sequence_id_to_bin_id[sequence_id] = predicted_bin
        except:
            exit("Error. File {} is malformed.".format(file_path_query))
    return query


def open_queries(file_path_queries):
    queries = []
    for file_path in file_path_queries:
        queries.append(open_query(file_path))
    return queries



#!/usr/bin/env python

import numpy as np


def filter_tail(data, percentage):
    value = float(percentage) / 100.0
    # sort bins by increasing predicted size
    data = sorted(data, key=lambda x: x['predicted_size'])
    sum_of_predicted_sizes = sum([int(float(bin['predicted_size'])) for bin in data])
    cumsum_of_predicted_sizes = 0
    for bin in data:
        predicted_size = int(float(bin['predicted_size']))
        cumsum_of_predicted_sizes += predicted_size
        if float(cumsum_of_predicted_sizes) / float(sum_of_predicted_sizes) <= value:
            bin['purity'] = np.nan
        else:
            break
    return data



#!/usr/bin/env python

import argparse
import collections
import os
import matplotlib
from version import __version__
from src import accuracy
from src import genome_recovery
from src import html_plots
from src import plot_by_genome
from src import plots
from src import precision_recall_by_bpcount
from src import precision_recall_per_bin
from src import rand_index
from src import precision_recall_average
matplotlib.use('Agg')
import matplotlib.pyplot as plt
from matplotlib.lines import Line2D
import pandas as pd
from src.utils import exclude_genomes
from src.utils import load_data
from src.utils import argparse_parents
from src.utils import labels


def get_labels(labels, bin_files):
    if labels:
        labels_list = [x.strip() for x in labels.split(',')]
        if len(labels_list) != len(bin_files):
            exit('Number of labels does not match the number of binning files. Please check parameter -l, --labels.')
        return labels_list
    tool_id = []
    for bin_file in bin_files:
        tool_id.append(bin_file.split('/')[-1])
    return tool_id


def evaluate_all(gold_standard_file,
                 fasta_file,
                 query_files,
                 binning_labels,
                 filter_tail_percentage,
                 filter_genomes_file,
                 keyword,
                 map_by_recall,
                 min_completeness, max_contamination,
                 output_dir):
    gold_standard = load_data.get_genome_mapping(gold_standard_file, fasta_file)
    labels_iterator = iter(binning_labels)
    summary_per_query = []
    bin_metrics_per_query = []
    count = 0
    for query_file in query_files:
        tool_id = query_file.split('/')[-1]
        binning_label = next(labels_iterator)
        path = os.path.join(output_dir, tool_id)
        load_data.make_sure_path_exists(path)

        f = open(os.path.join(path, "label.txt"), 'w')
        f.write("#({}){}".format(count, binning_label))
        f.close()

        query = load_data.open_query(query_file)

        if map_by_recall:
            bin_id_to_mapped_genome, bin_id_to_genome_id_to_total_length, mapped_genomes = precision_recall_per_bin.map_genomes_by_recall(gold_standard, query.bin_id_to_list_of_sequence_id)
        else:
            bin_id_to_mapped_genome, bin_id_to_genome_id_to_total_length, mapped_genomes = precision_recall_per_bin.map_genomes(gold_standard, query.bin_id_to_list_of_sequence_id)

        df_confusion = precision_recall_per_bin.compute_confusion_matrix(
            bin_id_to_mapped_genome,
            bin_id_to_genome_id_to_total_length,
            gold_standard,
            query)
        plots.plot_heatmap(df_confusion, path)

        # PRECISION RECALL PER BIN
        bin_metrics = precision_recall_per_bin.compute_metrics(query, gold_standard, bin_id_to_mapped_genome, bin_id_to_genome_id_to_total_length, mapped_genomes)
        if filter_genomes_file:
            bin_metrics = exclude_genomes.filter_data(bin_metrics, filter_genomes_file, keyword)
        f = open(os.path.join(path, "purity_completeness.tsv"), 'w')
        precision_recall_per_bin.print_metrics(bin_metrics, f)
        # slow code disabled
        # plot_by_genome.plot_by_genome(bin_metrics, path + '/genomes_sorted_by_recall', 'recall')
        # plot_by_genome.plot_by_genome(bin_metrics, path + '/genomes_sorted_by_precision', 'precision')
        f.close()

        # AVG PRECISION RECALL
        avg_precision, avg_recall, std_deviation_precision, std_deviation_recall, sem_precision, sem_recall = \
            precision_recall_average.compute_precision_and_recall(bin_metrics, filter_tail_percentage)
        f = open(os.path.join(path, "purity_completeness_avg.tsv"), 'w')
        precision_recall_average.print_precision_recall_table_header(f)
        precision_recall_average.print_precision_recall(binning_label,
                                                        avg_precision,
                                                        avg_recall,
                                                        std_deviation_precision,
                                                        std_deviation_recall,
                                                        sem_precision,
                                                        sem_recall,
                                                        f)
        f.close()

        # PRECISION RECALL BY BP COUNTS
        precision, recall = precision_recall_by_bpcount.compute_metrics(query, gold_standard)
        f = open(os.path.join(path, "purity_completeness_by_bpcount.tsv"), 'w')
        precision_recall_by_bpcount.print_precision_recall_by_bpcount(precision, recall, f)
        f.close()

        # (ADJUSTED) RAND INDEX
        ri_by_seq, ri_by_bp, a_rand_index_by_bp, a_rand_index_by_seq, percent_assigned_bps = rand_index.compute_metrics(query, gold_standard)
        f = open(os.path.join(path, "rand_index.tsv"), 'w')
        rand_index.print_rand_indices(ri_by_seq, ri_by_bp, a_rand_index_by_bp, a_rand_index_by_seq, percent_assigned_bps, f)
        f.close()

        # GENOME RECOVERY
        genome_recovery_val = genome_recovery.calc_dict(bin_metrics, min_completeness, max_contamination)

        # ACCURACY
        acc = accuracy.compute_metrics(query, gold_standard)

        summary_per_query.append(collections.OrderedDict([(labels.TOOL, binning_label),
                                                          (labels.AVG_PRECISION, avg_precision),
                                                          (labels.STD_DEV_PRECISION, std_deviation_precision),
                                                          (labels.SEM_PRECISION, sem_precision),
                                                          (labels.AVG_RECALL, avg_recall),
                                                          (labels.STD_DEV_RECALL, std_deviation_recall),
                                                          (labels.SEM_RECALL, sem_recall),
                                                          (labels.PRECISION, precision),
                                                          (labels.RECALL, recall),
                                                          (labels.RI_BY_BP, ri_by_bp),
                                                          (labels.RI_BY_SEQ, ri_by_seq),
                                                          (labels.ARI_BY_BP, a_rand_index_by_bp),
                                                          (labels.ARI_BY_SEQ, a_rand_index_by_seq),
                                                          (labels.PERCENTAGE_ASSIGNED_BPS, percent_assigned_bps),
                                                          (labels.ACCURACY, acc)] +
                                                         [(k, v) for k, v in genome_recovery_val.items()]))
        bin_metrics_per_query.append(bin_metrics)
        count += 1
    return summary_per_query, bin_metrics_per_query


def create_legend(summary_per_query, output_dir):
    colors_iter = iter(plots.create_colors_list())
    binning_labels = []
    circles = []
    for summary in summary_per_query:
        binning_labels.append(summary[labels.TOOL])
        circles.append(Line2D([], [], markeredgewidth=0.0, linestyle="None", marker="o", markersize=10, markerfacecolor=next(colors_iter)))

    fig = plt.figure(figsize=(0.5, 0.5))
    fig.legend(circles, binning_labels, loc='center', frameon=False, ncol=5, handletextpad=0.1)
    fig.savefig(os.path.normpath(output_dir + '/legend.eps'), dpi=100, format='eps', bbox_inches='tight')
    plt.close(fig)


def compute_rankings(summary_per_query, output_dir):
    f = open(os.path.normpath(output_dir + '/rankings.txt'), 'w')
    f.write("Tool\tAverage purity\n")
    sorted_by = sorted(summary_per_query, key=lambda x: x[labels.AVG_PRECISION], reverse=True)
    for summary in sorted_by:
        f.write("%s \t %1.3f\n" % (summary[labels.TOOL], summary[labels.AVG_PRECISION]))

    sorted_by = sorted(summary_per_query, key=lambda x: x[labels.AVG_RECALL], reverse=True)
    f.write("\nTool\tAverage completeness\n")
    for summary in sorted_by:
        f.write("%s \t %1.3f\n" % (summary[labels.TOOL], summary[labels.AVG_RECALL]))

    sorted_by = sorted(summary_per_query, key=lambda x: x[labels.AVG_PRECISION] + x[labels.AVG_RECALL], reverse=True)
    f.write("\nTool\tAverage purity + Average completeness\tAverage purity\tAverage completeness\n")
    for summary in sorted_by:
        f.write("%s\t%1.3f\t%1.3f\t%1.3f\n" % (summary[labels.TOOL],
                                               summary[labels.AVG_PRECISION] + summary[labels.AVG_RECALL],
                                               summary[labels.AVG_PRECISION],
                                               summary[labels.AVG_RECALL]))
    f.close()


def main():
    parser = argparse.ArgumentParser(description="Compute all metrics and figures for one or more binning files; output summary to screen and results per binning file to chosen directory",
                                     parents=[argparse_parents.PARSER_MULTI2], prog='AMBER')
    parser.add_argument('-o', '--output_dir', help="Directory to write the results to", required=True)
    parser.add_argument('-m', '--map_by_completeness', help=argparse_parents.HELP_MAP_BY_RECALL, action='store_true')
    parser.add_argument('-x', '--min_completeness', help=argparse_parents.HELP_THRESHOLDS_COMPLETENESS, required=False)
    parser.add_argument('-y', '--max_contamination', help=argparse_parents.HELP_THRESHOLDS_CONTAMINATION, required=False)
    parser.add_argument('-v', '--version', action='version', version='%(prog)s ' + __version__)
    args = parser.parse_args()

    min_completeness = None
    max_contamination = None
    if args.min_completeness:
        min_completeness = [int(x.strip())/100.0 for x in args.min_completeness.split(',')]
    if args.max_contamination:
        max_contamination = [int(x.strip())/100.0 for x in args.max_contamination.split(',')]

    binning_labels = get_labels(args.labels, args.bin_files)
    summary_per_query, bin_metrics_per_query = evaluate_all(args.gold_standard_file,
                                                            args.fasta_file,
                                                            args.bin_files,
                                                            binning_labels,
                                                            args.filter,
                                                            args.remove_genomes,
                                                            args.keyword,
                                                            args.map_by_completeness,
                                                            min_completeness, max_contamination,
                                                            args.output_dir)
    df = pd.DataFrame.from_dict(summary_per_query)
    print(df.to_csv(sep='\t', index=False, float_format='%.3f'))
    df.to_csv(path_or_buf=os.path.join(args.output_dir, "summary.tsv"), sep='\t', index=False, float_format='%.3f')

    create_legend(summary_per_query, args.output_dir)
    plots.plot_avg_precision_recall(summary_per_query, args.output_dir)
    plots.plot_weighed_precision_recall(summary_per_query, args.output_dir)
    plots.plot_adjusted_rand_index_vs_assigned_bps(summary_per_query, args.output_dir)

    plots.plot_boxplot(bin_metrics_per_query, binning_labels, 'purity', args.output_dir)
    plots.plot_boxplot(bin_metrics_per_query, binning_labels, 'completeness', args.output_dir)

    plot_by_genome.plot_by_genome2(bin_metrics_per_query, binning_labels, args.output_dir)
    compute_rankings(summary_per_query, args.output_dir)

    precision_recall_files = []
    for query_file in args.bin_files:
        tool_id = query_file.split('/')[-1]
        precision_recall_files.append(os.path.join(args.output_dir, tool_id, "purity_completeness.tsv"))
    df = pd.DataFrame.from_dict(summary_per_query)
    df.rename(columns={labels.TOOL: 'Tool'}, inplace=True)
    df.set_index('Tool', inplace=True)
    html_plots.build_html(precision_recall_files,
                          binning_labels,
                          df,
                          os.path.join(args.output_dir, "summary.html"))


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import argparse
import os, sys, inspect
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0, parentdir)
from src.utils import argparse_parents
from src.utils import load_data
from src.utils import labels


def choose2(n):
    return (n * (n - 1)) / 2.0


def compute_percentage_of_assigned_bps(query, gold_standard):
    assigned_bps = 0
    for bin_id in query.bin_id_to_list_of_sequence_id:
        for sequence_id in query.bin_id_to_list_of_sequence_id[bin_id]:
            assigned_bps += gold_standard.sequence_id_to_lengths[sequence_id]
    gold_size_in_bps = sum(gold_standard.genome_id_to_total_length.values())
    return float(assigned_bps) / float(gold_size_in_bps)


def preprocess_by_bp_counts(query, gold_standard):
    bin_id_to_genome_id_to_length = {}
    for bin_id in query.bin_id_to_list_of_sequence_id:
        bin_id_to_genome_id_to_length[bin_id] = {}
        for sequence_id in query.bin_id_to_list_of_sequence_id[bin_id]:
            genome_id = gold_standard.sequence_id_to_genome_id[sequence_id]
            if genome_id not in bin_id_to_genome_id_to_length[bin_id]:
                bin_id_to_genome_id_to_length[bin_id][genome_id] = 0
            bin_id_to_genome_id_to_length[bin_id][genome_id] += gold_standard.sequence_id_to_lengths[sequence_id]
    genome_id_to_bin_id_to_length = {}
    for sequence_id in query.sequence_id_to_bin_id:
        genome_id = gold_standard.sequence_id_to_genome_id[sequence_id]
        if genome_id not in genome_id_to_bin_id_to_length:
            genome_id_to_bin_id_to_length[genome_id] = {}
        bin_id = query.sequence_id_to_bin_id[sequence_id]
        if bin_id not in genome_id_to_bin_id_to_length[genome_id]:
            genome_id_to_bin_id_to_length[genome_id][bin_id] = 0
        genome_id_to_bin_id_to_length[genome_id][bin_id] += gold_standard.sequence_id_to_lengths[sequence_id]
    return bin_id_to_genome_id_to_length, genome_id_to_bin_id_to_length


def preprocess_by_sequence_counts(query, gold_standard):
    # metrics = precision_recall_average.filter_tail(metrics, 1)
    bin_id_to_genome_id_to_total_sequences = {}
    for bin_id in query.bin_id_to_list_of_sequence_id:
        bin_id_to_genome_id_to_total_sequences[bin_id] = {}
        for sequence_id in query.bin_id_to_list_of_sequence_id[bin_id]:
            genome_id = gold_standard.sequence_id_to_genome_id[sequence_id]
            # metric_entry = filter(lambda x: x['mapped_genome'] == genome_id, metrics)
            # if len(metric_entry) > 0 and np.isnan(metric_entry[0]['precision']):
            #     continue
            if genome_id not in bin_id_to_genome_id_to_total_sequences[bin_id]:
                bin_id_to_genome_id_to_total_sequences[bin_id][genome_id] = 0
            bin_id_to_genome_id_to_total_sequences[bin_id][genome_id] += 1
    genome_id_to_bin_id_to_total_sequences = {}
    for sequence_id in query.sequence_id_to_bin_id:
        genome_id = gold_standard.sequence_id_to_genome_id[sequence_id]
        # metric_entry = filter(lambda x: x['mapped_genome'] == genome_id, metrics)
        # if len(metric_entry) > 0 and np.isnan(metric_entry[0]['precision']):
        #     continue
        if genome_id not in genome_id_to_bin_id_to_total_sequences:
            genome_id_to_bin_id_to_total_sequences[genome_id] = {}
        bin_id = query.sequence_id_to_bin_id[sequence_id]
        if bin_id not in genome_id_to_bin_id_to_total_sequences[genome_id]:
            genome_id_to_bin_id_to_total_sequences[genome_id][bin_id] = 0
        genome_id_to_bin_id_to_total_sequences[genome_id][bin_id] += 1
    return bin_id_to_genome_id_to_total_sequences, genome_id_to_bin_id_to_total_sequences


def combinations(bin_id_to_genome_id_to_counts, genome_id_to_bin_id_to_counts):
    bin_genome_comb = 0.0
    bin_comb = 0.0
    genome_comb = 0.0
    total_counts = 0
    for bin_id in bin_id_to_genome_id_to_counts:
        for genome_id in bin_id_to_genome_id_to_counts[bin_id]:
            bin_genome_comb += choose2(bin_id_to_genome_id_to_counts[bin_id][genome_id])
    for bin_id in bin_id_to_genome_id_to_counts:
        bin_totals = 0
        for genome_id in bin_id_to_genome_id_to_counts[bin_id]:
            bin_totals += bin_id_to_genome_id_to_counts[bin_id][genome_id]
        total_counts += bin_totals
        bin_comb += choose2(bin_totals)
    for genome_id in genome_id_to_bin_id_to_counts:
        genome_totals = 0
        for bin_id in genome_id_to_bin_id_to_counts[genome_id]:
            genome_totals += genome_id_to_bin_id_to_counts[genome_id][bin_id]
        genome_comb += choose2(genome_totals)
    return bin_comb, genome_comb, bin_genome_comb, choose2(total_counts)


def compute_adjusted_rand_index(bin_id_to_genome_id_to_counts, genome_id_to_bin_id_to_counts):
    bin_comb, genome_comb, bin_genome_comb, num_bp_comb = combinations(bin_id_to_genome_id_to_counts, genome_id_to_bin_id_to_counts)
    temp = bin_comb * genome_comb / num_bp_comb
    ret = bin_genome_comb - temp
    return ret / (((bin_comb + genome_comb) / 2.0) - temp)


def compute_rand_index(bin_id_to_genome_id_to_counts, genome_id_to_bin_id_to_counts):
    bin_comb, genome_comb, bin_genome_comb, num_bp_comb = combinations(bin_id_to_genome_id_to_counts, genome_id_to_bin_id_to_counts)
    return (num_bp_comb - bin_comb - genome_comb + 2 * bin_genome_comb) / num_bp_comb


def print_rand_indices(ri_by_seq, ri_by_bp, ari_by_bp, ari_by_seq, percentage_of_assigned_bps, stream=sys.stdout):
    stream.write("%s\n%s\n" % ("\t".join((labels.RI_BY_BP, labels.RI_BY_SEQ, labels.ARI_BY_BP, labels.ARI_BY_SEQ, labels.PERCENTAGE_ASSIGNED_BPS)),
                 "\t".join((format(ri_by_bp, '.3f'),
                            format(ri_by_seq, '.3f'),
                            format(ari_by_bp, '.3f'),
                            format(ari_by_seq, '.3f'),
                            format(percentage_of_assigned_bps, '.3f')))))


def compute_metrics(query, gold_standard):
    # metrics = precision_recall_average.load_tsv_table(sys.stdin)
    bin_id_to_genome_id_to_total_sequences, genome_id_to_bin_id_to_total_sequences = preprocess_by_sequence_counts(query, gold_standard)
    bin_id_to_genome_id_to_length, genome_id_to_bin_id_to_length = preprocess_by_bp_counts(query, gold_standard)

    ri_by_seq = compute_rand_index(bin_id_to_genome_id_to_total_sequences, genome_id_to_bin_id_to_total_sequences)
    ri_by_bp = compute_rand_index(bin_id_to_genome_id_to_length, genome_id_to_bin_id_to_length)
    ari_by_seq = compute_adjusted_rand_index(bin_id_to_genome_id_to_total_sequences, genome_id_to_bin_id_to_total_sequences)
    ari_by_bp = compute_adjusted_rand_index(bin_id_to_genome_id_to_length, genome_id_to_bin_id_to_length)

    percentage_of_assigned_bps = compute_percentage_of_assigned_bps(query, gold_standard)

    return ri_by_seq, ri_by_bp, ari_by_bp, ari_by_seq, percentage_of_assigned_bps


def main():
    parser = argparse.ArgumentParser(description="Compute (adjusted) Rand index from binning file, unweighed and weighed by base pairs, and percentage of binned base pairs",
                                     parents=[argparse_parents.PARSER_GS])
    args = parser.parse_args()
    if not args.bin_file:
        parser.print_help()
        parser.exit(1)
    gold_standard = load_data.get_genome_mapping(args.gold_standard_file, args.fasta_file)
    query = load_data.open_query(args.bin_file)
    ri_by_seq, ri_by_bp, ari_by_bp, ari_by_seq, percentage_of_assigned_bps = compute_metrics(query, gold_standard)
    print_rand_indices(ri_by_seq, ri_by_bp, ari_by_bp, ari_by_seq, percentage_of_assigned_bps)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import argparse
import os, sys, inspect
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0, parentdir)
from src import precision_recall_per_bin
from src import precision_recall_average
from src.utils import argparse_parents
from src.utils import exclude_genomes
from src.utils import load_data


def evaluate_all(gold_standard, queries, labels, filter_tail_percentage, genomes_file, keyword, map_by_recall):
    precision_recall_average.print_precision_recall_table_header()
    labels_iterator = iter(labels)
    for query in queries:

        bin_metrics = precision_recall_per_bin.compute_metrics(query, gold_standard, map_by_recall)

        if genomes_file:
            bin_metrics = exclude_genomes.filter_data(bin_metrics, genomes_file, keyword)

        avg_precision, avg_recall, std_deviation_precision, std_deviation_recall, std_error_precision, std_error_recall = \
            precision_recall_average.compute_precision_and_recall(bin_metrics, filter_tail_percentage)

        precision_recall_average.print_precision_recall(next(labels_iterator) if len(labels) > 0 else query.path.split('/')[-1],
                                                        avg_precision,
                                                        avg_recall,
                                                        std_deviation_precision,
                                                        std_deviation_recall,
                                                        std_error_precision,
                                                        std_error_recall)


def main():
    parser = argparse.ArgumentParser(description="Compute precision and recall, including standard deviation and standard error of the mean, for binning files",
                                     parents=[argparse_parents.PARSER_MULTI2])
    parser.add_argument('-m', '--map_by_recall', help=argparse_parents.HELP_MAP_BY_RECALL, action='store_true')
    args = parser.parse_args()
    labels = []
    if args.labels:
        labels = [x.strip() for x in args.labels.split(',')]
        if len(labels) != len(args.bin_files):
            parser.error('number of labels does not match the number of query files')
    gold_standard = load_data.get_genome_mapping(args.gold_standard_file, args.fasta_file)
    queries = load_data.open_queries(args.bin_files)
    evaluate_all(gold_standard,
                 queries,
                 labels,
                 args.filter,
                 args.remove_genomes,
                 args.keyword,
                 args.map_by_recall)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import argparse
import math
import numpy as np
import os, sys, inspect
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0, parentdir)
from src.utils import argparse_parents
from src.utils import filter_tail
from src.utils import load_data

from src.utils import labels


def compute_precision_and_recall(data, filter_tail_percentage):
    avg_precision = .0
    avg_recall = .0
    count_p = 0
    count_r = 0

    if filter_tail_percentage:
        filter_tail.filter_tail(data, filter_tail_percentage)

    # for each predicted bin (row of the table)
    for bin in data:
        # compute the average recall over all bins if the mapped genome size > 0
        real_size = float(bin['real_size'])
        if real_size > 0:
            recall = float(bin['completeness'])
            count_r += 1
            current_avg = avg_recall
            avg_recall = (recall - current_avg) / count_r + current_avg

        # compute the average precision over all bins
        if not np.isnan(bin['purity']):
            precision = bin['purity']
            count_p += 1
            current_avg = avg_precision
            avg_precision = (precision - current_avg) / count_p + current_avg

    sum_diffs_precision = .0
    sum_diffs_recall = .0
    for bin in data:
        real_size = float(bin['real_size'])
        if real_size > 0:
            recall = float(bin['completeness'])
            sum_diffs_recall += math.pow(recall - avg_recall, 2)
        if not np.isnan(bin['purity']):
            precision = bin['purity']
            sum_diffs_precision += math.pow(precision - avg_precision, 2)

    std_deviation_precision = math.sqrt(sum_diffs_precision / count_p)
    std_error_precision = std_deviation_precision / math.sqrt(count_p)

    std_deviation_recall = math.sqrt(sum_diffs_recall / count_r)
    std_error_recall = std_deviation_recall / math.sqrt(count_r)

    return avg_precision, avg_recall, std_deviation_precision, std_deviation_recall, std_error_precision, std_error_recall


def print_precision_recall_table_header(stream=sys.stdout):
    stream.write("%s\n" % "\t".join((labels.TOOL, labels.AVG_PRECISION, labels.STD_DEV_PRECISION,
                                     labels.SEM_PRECISION, labels.AVG_RECALL, labels.STD_DEV_RECALL, labels.
                                     SEM_RECALL)))


def print_precision_recall(label, avg_precision, avg_recall, std_deviation_precision, std_deviation_recall, std_error_precision, std_error_recall,
                           stream=sys.stdout):
    if not label:
        label = ""
    stream.write("%s\n" % "\t".join((label,
                                    format(avg_precision, '.3f'),
                                    format(std_deviation_precision, '.3f'),
                                    format(std_error_precision, '.3f'),
                                    format(avg_recall, '.3f'),
                                    format(std_deviation_recall, '.3f'),
                                    format(std_error_recall, '.3f'))))


def main():
    parser = argparse.ArgumentParser(description="Compute precision and recall, including standard deviation and standard error of the mean, from table of precision and recall per genome. The table can be provided as file or via the standard input",
                                     parents=[argparse_parents.PARSER_MULTI])
    args = parser.parse_args()
    if not args.file and sys.stdin.isatty():
        parser.print_help()
        parser.exit(1)
    metrics = load_data.load_tsv_table(sys.stdin if not sys.stdin.isatty() else args.file)
    avg_precision, avg_recall, std_deviation_precision, std_deviation_recall, std_error_precision, std_error_recall =\
        compute_precision_and_recall(metrics, args.filter)
    print_precision_recall_table_header()
    print_precision_recall(args.label, avg_precision, avg_recall, std_deviation_precision, std_deviation_recall, std_error_precision, std_error_recall)

if __name__ == "__main__":
    main()



#!/usr/bin/env python

import argparse
from collections import Counter
from collections import defaultdict
import os, sys, inspect
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0, parentdir)
from src.utils import argparse_parents
from src.utils import load_data


def calc_precision_recall(bin_id_to_genome_id_to_total_length, bin_id_to_total_lengths, genome_id_to_total_length):
    # now calculate precision looping bins
    true_positives_p = 0
    denominator_p = sum(bin_id_to_total_lengths.values())

    for predicted_bin, genome_assigns in list(bin_id_to_genome_id_to_total_length.items()):
        # get maximal genome assignment
        if len(genome_assigns) > 0:
            maxAssign = max(genome_assigns.values())
        else:
            maxAssign = 0.0

        true_positives_p += maxAssign

    # now calculate precision as TP/(FP + TP)
    precision = float(true_positives_p) / float(denominator_p)

    # now calculate recall looping genomes
    true_positives_r = 0.0
    for genome_id in genome_id_to_total_length:
        # now loop bins
        bin_assigns = []
        for bin_id in bin_id_to_total_lengths:
            if genome_id in bin_id_to_genome_id_to_total_length[bin_id]:
                bin_assigns.append(bin_id_to_genome_id_to_total_length[bin_id][genome_id])
        if len(bin_assigns) > 0:
            maxAssign = max(bin_assigns)
        else:
            maxAssign = 0.0
        true_positives_r += maxAssign

    denominator_r = sum(genome_id_to_total_length.values())
    recall = true_positives_r / denominator_r

    return precision, recall


def compute_metrics(query, gold_standard):
    """
    @param query
    @param gold_standard
    @return: query, gold_standard
    """
    bin_id_to_total_length = {}
    bin_id_to_genome_id_to_total_length = defaultdict(Counter)
    for predicted_bin in query.bin_id_to_list_of_sequence_id:
        if predicted_bin not in bin_id_to_total_length:
            bin_id_to_total_length[predicted_bin] = 0
        for sequence_id in query.bin_id_to_list_of_sequence_id[predicted_bin]:
            bin_id_to_total_length[predicted_bin] += gold_standard.sequence_id_to_lengths[sequence_id]
            genome_id = gold_standard.sequence_id_to_genome_id[sequence_id]
            bin_id_to_genome_id_to_total_length[predicted_bin][genome_id] += gold_standard.sequence_id_to_lengths[sequence_id]
    precision, recall = calc_precision_recall(bin_id_to_genome_id_to_total_length, bin_id_to_total_length,
                                              gold_standard.genome_id_to_total_length)
    return precision, recall


def print_precision_recall_by_bpcount(precision, recall, stream=sys.stdout):
    stream.write("precision\trecall\n%1.3f\t%1.3f\n" % (precision, recall))


def main():
    parser = argparse.ArgumentParser(description="Compute precision and recall weighed by base pair counts (not averaged over genome bins) from binning file",
                                     parents=[argparse_parents.PARSER_GS])
    args = parser.parse_args()
    if not args.bin_file:
        parser.print_help()
        parser.exit(1)
    gold_standard = load_data.get_genome_mapping(args.gold_standard_file, args.fasta_file)
    query = load_data.open_query(args.bin_file)
    precision, recall = compute_metrics(query, gold_standard)
    print_precision_recall_by_bpcount(precision, recall)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import argparse
import os
import sys

import matplotlib
import numpy as np

matplotlib.use('Agg')
import matplotlib.pyplot as plt
import os, sys, inspect
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0, parentdir)
from src import plots
from src.utils import load_data
from src.utils import argparse_parents


def plot_by_genome(data, out_file=None, sort_by='completeness'):
    not_sort_by = list(set(['purity','completeness']) - set([sort_by]))[0]  # get the metric not sorted by
    data = sorted(data, key=lambda x: x[sort_by])
    genomes = []
    precision = []
    recall = []
    for genome in data:
        genomes.append(genome['mapped_genome'])
        precision.append(genome['purity'])
        recall.append(genome['completeness'])
    sort = {'purity': precision, 'completeness': recall}

    fig, ax1 = plt.subplots(figsize=(len(genomes) * 0.15, 5))

    ax1.plot(np.arange(len(genomes)), sort[sort_by], color='black')
    plt.xticks(np.arange(len(genomes)), genomes, rotation='vertical', fontsize="smaller")
    ax1.plot(np.arange(len(genomes)), sort[not_sort_by], '.', color='red')

    # transform y labels to percentages
    vals = ax1.get_yticks()
    ax1.set_yticklabels(['{:3.0f}%'.format(x * 100) for x in vals])

    plt.legend((sort_by.title(), not_sort_by.title()))
    plt.grid(True)
    plt.tight_layout()
    if out_file is None:
        plt.show()
    else:
        plt.savefig(os.path.normpath(out_file + '.png'), dpi=100, format='png', bbox_inches='tight')
        plt.savefig(os.path.normpath(out_file + '.pdf'), dpi=100, format='pdf', bbox_inches='tight')
    plt.close(fig)


def plot_by_genome2(bin_metrics_per_query, binning_labels, output_dir):
    colors_list = plots.create_colors_list()
    if len(bin_metrics_per_query) > len(colors_list):
        raise RuntimeError("Plot only supports 29 colors")

    fig, axs = plt.subplots(figsize=(6, 5))

    # force axis to be from 0 to 100%
    axs.set_xlim([0.0, 1.0])
    axs.set_ylim([0.0, 1.0])

    i = 0
    for query_metrics in bin_metrics_per_query:
        precision = []
        recall = []
        for metrics in query_metrics:
            precision.append(metrics['purity'])
            recall.append(metrics['completeness'])
        axs.scatter(precision, recall, marker='o', color=colors_list[i], s=[8] * len(precision))
        i += 1

    # turn on grid
    axs.minorticks_on()
    axs.grid(which='major', linestyle='-', linewidth='0.5')
    axs.grid(which='minor', linestyle=':', linewidth='0.5')

    # transform plot_labels to percentages
    vals = axs.get_xticks()
    axs.set_xticklabels(['{:3.0f}%'.format(x * 100) for x in vals])
    vals = axs.get_yticks()
    axs.set_yticklabels(['{:3.0f}%'.format(x * 100) for x in vals])

    plt.xlabel('Purity per bin')
    plt.ylabel('Completeness per genome')
    plt.tight_layout()
    fig.savefig(os.path.normpath(output_dir + '/purity_completeness_per_bin.eps'), dpi=100, format='eps', bbox_inches='tight')

    lgd = plt.legend(binning_labels, bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0., handlelength=0, frameon=False)
    for handle in lgd.legendHandles:
        handle.set_sizes([100.0])

    fig.savefig(os.path.normpath(output_dir + '/purity_completeness_per_bin.png'), dpi=100, format='png', bbox_extra_artists=(lgd,), bbox_inches='tight')
    fig.savefig(os.path.normpath(output_dir + '/purity_completeness_per_bin.pdf'), dpi=100, format='pdf', bbox_extra_artists=(lgd,), bbox_inches='tight')
    plt.close(fig)


def main():
    parser = argparse.ArgumentParser(description="Plot purity and completeness per genome. Genomes can be sorted by completeness (default) or purity")
    parser.add_argument('file', nargs='?', type=argparse.FileType('r'), help=argparse_parents.HELP_FILE)
    parser.add_argument('-s','--sort_by', help='Sort by either purity or completeness (default: completeness)', choices=set(['purity','completeness']))
    parser.add_argument('-o','--out_file', help='Path to store image (default: only show image)')
    args = parser.parse_args()
    if not args.file and sys.stdin.isatty():
        parser.print_help()
        parser.exit(1)
    metrics = load_data.load_tsv_table(sys.stdin if not sys.stdin.isatty() else args.file)
    if args.sort_by is not None:
        plot_by_genome(metrics, args.out_file, args.sort_by)
    else:
        plot_by_genome(metrics, args.out_file)
        

if __name__ == "__main__":
    main()



#!/usr/bin/env python

import argparse
import datetime
import re
from math import pi
from collections import OrderedDict

import bokeh.models.widgets.tables
import numpy as np
import pandas as pd
from bokeh import palettes
from bokeh.embed import file_html
from bokeh.layouts import widgetbox, layout
from bokeh.models import (ColorBar,
                          Text,
                          BasicTicker,
                          HoverTool,
                          FuncTickFormatter,
                          DataTable,
                          widgets)
from bokeh.models.mappers import LinearColorMapper
from bokeh.models.widgets import Div
from bokeh.plotting import figure, ColumnDataSource
from bokeh.resources import CDN

import os, sys, inspect
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0, parentdir)
from src.utils import labels
from version import __version__

SCATTER_ELEMENT_WIDTH = 1500
SCATTER_ELEMENT_HEIGHT = 500

TABLE_ELEMENT_WIDTH = 750
TABLE_ELEMENT_HEIGHT = 300

DIV_WIDTH = 1200
DIV_HEIGHT = None

DESCRIPTION_WIDTH = 600
DESCRIPTION_HEIGHT = 60

A_WIDTH = 1200
A_HEIGHT = 15

COLORS_20 = palettes.d3['Category20'][20]
COLORS_10 = palettes.d3['Category10'][10]
HEATMAP_COLORS = list(reversed(palettes.RdYlBu[10])) + ['white']

NO_COLOR_COLUMNS = [labels.SEM_PRECISION, labels.SEM_RECALL, labels.STD_DEV_PRECISION, labels.STD_DEV_RECALL]

COL_DESCRIPTION = "description"
COL_TITLE = "title"
COL_FILE_ID = "file_id"

ID_SUMMARY = "summary"
ID_CONTAMINATION_COMPLETENESS = "contamination_completeness"
ID_PRECISION_VS_RECALL_TOOLS = "purity_vs_completeness_tools"
ID_PRECISION_VS_RECALL_TOOLS_BASE_PAIR = "purity_vs_completeness_tools_base_pair"
ID_RAND_INDEX_ASSIGNED_BPS = "rand_index_assigned_bps"
ID_ALL_GENOMES = "all_genomes"
ID_SINGLE_TOOL = "single_tool"

DESCRIPTION_PRECISION_RECALL_TOOLS = """
Scatter plot showing the average purity per bin vs. the average completeness per genome including the standard error
of the mean.
"""

DESCRIPTION_PRECISION_RECALL_TOOLS_BASE_PAIR = """
Scatter plot showing the average purity vs. the average completeness per base pair.
"""

DESCRIPTION_PRECISION_RECALL_GENOME = """
Scatter plot showing purity vs. completeness per genome per tool.
"""

DESCRIPTION_ADJUSTED_RAND_INDEX_TOOLS = """
Scatter plot showing adjusted rand index vs percentage of assigned base pairs.
"""


def get_color(number_of_tools):
    if number_of_tools > 10:
        return COLORS_20
    else:
        return COLORS_10


def create_title_div(id, name, info):
    div = Div(text="""<h1 id="{0}">{1}</h1><p>{2}</p>""".format(id, name, info),
              width=DIV_WIDTH, height=DIV_HEIGHT)
    return [div]


def _set_default_figure_properties(figure, x_label, y_label):
    figure.legend.click_policy = "hide"
    figure.legend.location = "top_right"
    figure.axis.axis_label_text_font_size = "15pt"
    figure.axis.major_label_text_font_size = "13pt"
    figure.xaxis.axis_label = x_label
    figure.yaxis.axis_label = y_label
    figure.xaxis.major_label_orientation = "vertical"
    return figure


def create_description(content):
    div = Div(text="""<h3>Description:</h3><p>{0}</p>""".format(content))
    return [div]


def create_subtitle_div(id, name):
    div = Div(text="""<h2 id="{0}">{1}</h2>""".format(id, name),
              width=DIV_WIDTH, height=DIV_HEIGHT)
    return [div]


def create_subtitle_a(href, name):
    div = Div(text="""<a href=#{0} style='padding-left:30px'>{1}</h2>""".format(href, name),
              width=A_WIDTH, height=A_HEIGHT)
    return [div]


def create_title_a(href, number, name):
    div = Div(text="""<a href=#{0}>{2}. {1}</h2>""".format(href, name, number),
              width=A_WIDTH, height=A_HEIGHT)
    return [div]


def get_contamination_completeness_thresholds(df):
    contamination_completeness_cols = [c for c in df.columns]
    completeness_thr = []
    contamination_thr = []
    for col in contamination_completeness_cols:
        values = col.split('<')
        completeness_item = values[0]
        contamination_item = '<' + values[1]
        if completeness_item not in completeness_thr:
            completeness_thr.append(completeness_item)
        if contamination_item not in contamination_thr:
            contamination_thr.append(contamination_item)
    return completeness_thr, contamination_thr


def create_contamination_completeness_table(df, completeness_thr, contamination_thr):
    contamination_completenes_row_arr = []
    for index, row in df.iterrows():
        for contamination_item in contamination_thr:
            cols = OrderedDict()
            contamination_val = format(float(re.match("\d+\.\d+", contamination_item[1:]).group()) * 100, '.0f')
            cols['Tool'] = '{} <{}% contamination'.format(row.name, contamination_val)
            for completeness_item in completeness_thr:
                cols[completeness_item] = row['{}{}'.format(completeness_item, contamination_item)]
            contamination_completenes_row_arr.append(cols)

    df = pd.DataFrame(contamination_completenes_row_arr)

    def create_table_column(field):
        if field == "Tool":
            return widgets.TableColumn(title=field, field=field, width=600)
        else:
            completeness_val = format(float(re.match("\d+\.\d+", field[1:]).group()) * 100, '.0f')
            return widgets.TableColumn(title='>{}% completeness'.format(completeness_val), field=field)

    dt = DataTable(source=bokeh.models.ColumnDataSource(df),
                   columns=list(map(lambda x: create_table_column(x), df.columns.values)),
                   width=TABLE_ELEMENT_WIDTH,
                   reorderable=True,
                   selectable=True)
    return [widgetbox(dt)]


def create_summary_heatmap(df, std_dev_sem_columns):
    df = df.set_index('Tool').iloc[::-1]
    tools = list(df.index)
    metrics = list(reversed(list(df.columns)))

    # unweighted columns should be at the right side
    for column in std_dev_sem_columns:
        metrics.append(metrics.pop(metrics.index(column)))

    df = df[metrics]

    UNWEIGHTED_NUMBER = 1.10001
    WEIGHTING_COLUMN = 'rate_extended'
    DEFAULT_TOOL_HEIGHT = 10
    COLORBAR_HEIGHT = 150
    ALPHA_COLOR=0.85

    df.columns.name = 'Metrics'
    df = pd.DataFrame(df.stack(), columns=['rate']).reset_index()
    df['rate'] = df['rate'].map('{:,.5f}'.format)
    df[WEIGHTING_COLUMN] = df['rate']

    for column in std_dev_sem_columns:
        df.loc[df.Metrics == column, WEIGHTING_COLUMN] = UNWEIGHTED_NUMBER

    mapper = LinearColorMapper(palette=HEATMAP_COLORS, low=0, high=UNWEIGHTED_NUMBER)
    source = ColumnDataSource(df)

    p = figure(x_range=metrics, y_range=tools,
               x_axis_location="above", plot_height=len(tools) * DEFAULT_TOOL_HEIGHT + COLORBAR_HEIGHT,
               tools="hover,save,box_zoom,reset,wheel_zoom", toolbar_location='below')

    p = _set_default_figure_properties(p, "Metrics", "Tools")
    p.xaxis.major_label_orientation = pi / 2.5

    p.rect(x="Metrics", y="Tool",
           width=1,
           height=1,
           source=source,
           alpha=ALPHA_COLOR,
           fill_color={'field': WEIGHTING_COLUMN, 'transform': mapper},
           line_color="black")

    glyph = Text(x="Metrics", y="Tool", text_align="center",
                 text_font_size="10pt",
                 text_baseline="middle", text="rate", text_color="black")
    p.add_glyph(source, glyph)

    tickFormatter = FuncTickFormatter(code="""
    if(tick==1){
        return tick + " (good)"
    } else if(tick==0){
        return tick + " (bad)"
    } else if(tick==1.1){
        return "Unweighted"
    } else {
        return tick.toLocaleString(
  undefined, // use a string like 'en-US' to override browser locale
  { minimumFractionDigits: 2 }
);
    }
    """)

    color_bar = ColorBar(color_mapper=mapper,
                         major_label_text_font_size="12pt",
                         ticker=BasicTicker(desired_num_ticks=len(HEATMAP_COLORS)),
                         scale_alpha=ALPHA_COLOR,
                         major_label_text_align="right",
                         major_label_text_baseline="middle",
                         bar_line_color="black",
                         formatter=tickFormatter,
                         label_standoff=13,
                         orientation="horizontal",
                         location=(-250, 0))

    p.add_layout(color_bar, 'above')
    p.select_one(HoverTool).tooltips = [
        ('Metric', '@Metrics'),
        ('Tool', '@Tool'),
        ('Value', '@rate'),
    ]
    return [p]


def errorbar(fig, x, y, xerr=None, yerr=None, color='red',
             point_kwargs={}, error_kwargs={}):
    """
    Errorbar based on
    https://stackoverflow.com/questions/29166353/how-do-you-add-error-bars-to-bokeh-plots-in-python
    :param fig: Bokeh figure
    :param x: x coordinate
    :param y: y coordinate
    :param xerr: x axis error bar
    :param yerr: y axis error bar
    :param color: color of the error bar
    :param point_kwargs: further arguments provided to bokeh circle function
    :param error_kwargs: further arguments provided to bokeh multiline function
    """
    fig.circle(x, y, color=color, **point_kwargs)

    if xerr:
        x_err_x = []
        x_err_y = []
        for px, py, err in zip(x, y, xerr):
            x_err_x.append((px - err, px + err))
            x_err_y.append((py, py))
        fig.multi_line(x_err_x, x_err_y, color=color, **error_kwargs)

    if yerr:
        y_err_x = []
        y_err_y = []
        for px, py, err in zip(x, y, yerr):
            y_err_x.append((px, px))
            y_err_y.append((py - err, py + err))
        fig.multi_line(y_err_x, y_err_y, color=color, **error_kwargs)


def create_scatter(df, x, y, x_title, y_title):
    """
    Creates scatter plot.
    :param df: Dataframe
    :param x: column name for x variable
    :param y: column name for y variable
    :param x_title: Title for x axis
    :param y_title: Title for y axis
    :return: bokeh figure
    """
    p = figure(plot_width=SCATTER_ELEMENT_WIDTH, plot_height=SCATTER_ELEMENT_HEIGHT)

    for i, (idx, row) in zip(np.arange(len(df.index)), df.iterrows()):
        p.circle(row[x], row[y], alpha=0.8, color=get_color(len(df.index))[i],
                 size=10,
                 legend=idx)

    p = _set_default_figure_properties(p, x_title, y_title)

    return [p]


def create_precision_recall_all_genomes_scatter(paths, names):
    """
    Creates purity vs completeness scatter plot for all tools and genomes.
    :param paths: Path to all purity and completeness tables
    :return: bokeh figure
    """

    def assign_tool_name(path, name):
        df = pd.DataFrame.from_csv(path, sep='\t', header=0)
        df["name"] = name
        df["genome"] = df.index
        return df

    df = pd.concat(map(lambda x: assign_tool_name(*x), zip(paths, names)))

    p = figure(plot_width=SCATTER_ELEMENT_WIDTH, plot_height=SCATTER_ELEMENT_HEIGHT)
    p.add_tools(HoverTool(tooltips=[
        ("genome", "@genome")
    ]))
    names = list(df.name.unique())

    def create_circles(df, plot, name, color):
        df = df[df["name"] == name]
        source = ColumnDataSource(data=df)
        plot.circle("purity", "completeness", color=color, alpha=0.8, legend=name, source=source)
        return plot

    for idx, name in enumerate(names):
        p = create_circles(df, p, name, get_color(len(names))[idx])

    p = _set_default_figure_properties(p, "Purity", "Completeness")
    return [p]


def save_html_file(path, elements):
    html = file_html(layout(elements, sizing_mode='scale_width'), CDN, "AMBER: Assessment of Metagenome BinnERs")
    file = open(path, "w+")
    file.write(html)
    file.close()


def build_html_summary_path(precision_recall_paths, names, summary, html_output):
    summary_df = pd.DataFrame.from_csv(summary, sep='\t', header=0)
    build_html(precision_recall_paths, names, summary_df, html_output, NO_COLOR_COLUMNS)


def build_html(precision_recall_paths, names, summary, html_output, std_dev_sem_columns=NO_COLOR_COLUMNS):
    contamination_completeness_cols = [c for c in summary.columns if c.startswith('>')]
    element_column = list()

    summary.insert(0, "Tool", summary.index)
    element_column.append(
        create_title_div("main", "AMBER: Assessment of Metagenome BinnERs", " produced on {0} with AMBER version {1} ".format(
            datetime.datetime.now().strftime("%Y-%m-%d %H:%M"), __version__)))
    element_column.append(create_subtitle_div("contents", "Contents"))

    element_column.append(create_title_a(ID_SUMMARY, "1", "Summary"))
    element_column.append(create_title_a(ID_CONTAMINATION_COMPLETENESS, "2", "Contamination and completeness"))
    element_column.append(create_title_a(ID_PRECISION_VS_RECALL_TOOLS, "3", "Average purity per bin vs. average completeness per genome"))
    element_column.append(create_title_a(ID_PRECISION_VS_RECALL_TOOLS_BASE_PAIR, "4", "Average purity vs. average completeness per base pair"))
    element_column.append(
        create_title_a(ID_RAND_INDEX_ASSIGNED_BPS, "5", "Adjusted Rand Index vs. percentage of assigned base pairs"))
    element_column.append(create_title_a(ID_ALL_GENOMES, "6", "Purity vs. completeness of all genome bins"))

    element_column.append(create_subtitle_div(ID_SUMMARY, "Summary"))

    def create_entry(k, v):
        return "<li><strong>{0}: </strong>{1}</li>".format(k, v)

    html_listing = list(map(lambda k: create_entry(k, labels.abbreviations[k]),
                            list(filter(lambda k: k not in contamination_completeness_cols,
                                        labels.abbreviations.keys()))))

    element_column.append(
        create_description("<ul>{0}</ul>".format(" ".join(html_listing))))
    df_without_contam_complete = summary.drop(contamination_completeness_cols, axis=1)

    element_column.append(create_summary_heatmap(df_without_contam_complete, std_dev_sem_columns))

    completeness_thr, contamination_thr = get_contamination_completeness_thresholds(summary[contamination_completeness_cols])

    completeness_thr_formatted = []
    for completeness_item in completeness_thr:
        value = format(float(re.match("\d+\.\d+", completeness_item[1:]).group()) * 100, '.0f')
        completeness_thr_formatted.append('{}% completeness'.format(value))

    contamination_thr_formatted = []
    for contamination_item in contamination_thr:
        value = format(float(re.match("\d+\.\d+", contamination_item[1:]).group()) * 100, '.0f')
        contamination_thr_formatted.append('{}% contamination'.format(value))

    def create_entry_completeness(v):
        return "<li><strong>>{0}: </strong>number of bins with more than {1}</li>".format(v, v)

    def create_entry_contamination(v):
        return "<li><strong><{0}: </strong>number of bins with less than {1}</li>".format(v, v)

    completion_contamination_col = [create_entry_completeness(v) for v in completeness_thr_formatted]
    completion_contamination_row = [create_entry_contamination(v) for v in contamination_thr_formatted]

    element_column.append(create_subtitle_div(ID_CONTAMINATION_COMPLETENESS, "Contamination and completeness"))

    element_column.append(
        create_description("Table columns: <ul>{0}</ul> "
                           "Table rows: <ul>{1}</ul>".format(" ".join(completion_contamination_col),
                                                             " ".join(completion_contamination_row))))
    element_column.append(create_contamination_completeness_table(summary[contamination_completeness_cols], completeness_thr, contamination_thr))

    element_column.append(create_subtitle_div(ID_PRECISION_VS_RECALL_TOOLS, "Average purity per bin vs. average completeness per genome"))
    element_column.append(create_description(DESCRIPTION_PRECISION_RECALL_TOOLS))
    element_column.append(create_scatter(summary, "avg_purity", "avg_completeness", "Average purity per bin", "Average completeness per genome"))

    element_column.append(create_subtitle_div(ID_PRECISION_VS_RECALL_TOOLS_BASE_PAIR, "Average purity per base pair vs. average completeness per base pair"))
    element_column.append(create_description(DESCRIPTION_PRECISION_RECALL_TOOLS_BASE_PAIR))
    element_column.append(create_scatter(summary, "avg_purity_per_bp", "avg_completeness_per_bp", "Average purity per base pair", "Average completeness per base pair"))

    element_column.append(
        create_subtitle_div(ID_RAND_INDEX_ASSIGNED_BPS, "Adjusted Rand Index vs. percentage of assigned base pairs"))
    element_column.append(create_description(DESCRIPTION_ADJUSTED_RAND_INDEX_TOOLS))
    element_column.append(create_scatter(summary, "a_rand_index_by_bp", "percent_assigned_bps",
                                         "Adjusted Rand Index by base pair", "Percentage of assigned base pairs"))

    element_column.append(create_subtitle_div(ID_ALL_GENOMES, "Purity vs. completeness of all genome bins"))
    element_column.append(create_description(DESCRIPTION_PRECISION_RECALL_GENOME))
    element_column.append(create_precision_recall_all_genomes_scatter(precision_recall_paths, names))

    save_html_file(html_output, element_column)


def main():
    parser = argparse.ArgumentParser(description="Create HTML-based plots.")
    parser.add_argument('-o', '--output_file', help="Directory to write the results to", required=True)
    parser.add_argument('-p', '--purity_completeness_files', nargs='+', help='<Required> Set flag', required=True)
    parser.add_argument('-n', '--names', nargs='+', help='<Required> Set flag', required=True)
    parser.add_argument('-s', '--summary', help='Summary of all metrics', required=True)
    args = parser.parse_args()
    build_html_summary_path(args.purity_completeness_files, args.names, args.summary, args.output_file)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import argparse
from collections import Counter
from collections import defaultdict
import os, sys, inspect
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0, parentdir)
from src.utils import argparse_parents
from src.utils import load_data


def calc_accuracy(bin_id_to_genome_id_to_total_length, gold_standard):
    # query_sequence_ids = set(query.sequence_id_to_bin_id.keys())
    # gs_sequence_ids = set(gold_standard.sequence_id_to_lengths.keys())
    # unassigned = 0
    # for unassigned_seq_id in gs_sequence_ids - query_sequence_ids:
    #     unassigned += gold_standard.sequence_id_to_lengths[unassigned_seq_id]
    # denominator_p = sum(bin_id_to_total_lengths.values()) + unassigned

    # summing up assigned and unassigned bps is equivalent to summing up the length of all genomes
    denominator_p = 0
    for genome_id in gold_standard.genome_id_to_total_length.keys():
        denominator_p += gold_standard.genome_id_to_total_length[genome_id]

    true_positives_p = 0
    for predicted_bin, genome_assigns in list(bin_id_to_genome_id_to_total_length.items()):
        # get maximal genome assignment
        if len(genome_assigns) > 0:
            max_assign = max(genome_assigns.values())
        else:
            max_assign = 0.0
        true_positives_p += max_assign

    # calculate accuracy as TP/(FP + TP + U)
    accuracy = float(true_positives_p) / float(denominator_p)
    return accuracy


def compute_metrics(query, gold_standard):
    """
    @param query
    @param gold_standard
    @return: query, gold_standard
    """
    bin_id_to_total_length = {}
    bin_id_to_genome_id_to_total_length = defaultdict(Counter)
    for predicted_bin in query.bin_id_to_list_of_sequence_id:
        if predicted_bin not in bin_id_to_total_length:
            bin_id_to_total_length[predicted_bin] = 0
        for sequence_id in query.bin_id_to_list_of_sequence_id[predicted_bin]:
            bin_id_to_total_length[predicted_bin] += gold_standard.sequence_id_to_lengths[sequence_id]
            genome_id = gold_standard.sequence_id_to_genome_id[sequence_id]
            bin_id_to_genome_id_to_total_length[predicted_bin][genome_id] += gold_standard.sequence_id_to_lengths[sequence_id]
    accuracy = calc_accuracy(bin_id_to_genome_id_to_total_length, gold_standard)
    return accuracy


def main():
    parser = argparse.ArgumentParser(description="Compute accuracy from binning file",
                                     parents=[argparse_parents.PARSER_GS])
    args = parser.parse_args()
    if not args.bin_file:
        parser.print_help()
        parser.exit(1)
    gold_standard = load_data.get_genome_mapping(args.gold_standard_file, args.fasta_file)
    query = load_data.open_query(args.bin_file)
    accuracy = compute_metrics(query, gold_standard)
    print(accuracy)

if __name__ == "__main__":
    main()



#!/usr/bin/env python

import argparse
import numpy as np
import collections
import os, sys, inspect
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0, parentdir)
from src.utils import argparse_parents
from src.utils import filter_tail
from src.utils import load_data


def get_defaults():
    return [.5, .7, .9], [.1, .05]


def calc_table(metrics, min_completeness, max_contamination):
    if not min_completeness:
        min_completeness = get_defaults()[0]
    if not max_contamination:
        max_contamination = get_defaults()[1]

    genome_recovery = np.zeros((len(max_contamination), len(min_completeness)), dtype=int)
    for metric in metrics:
        precision = float(metric['purity'])
        recall = float(metric['completeness'])
        if np.isnan(precision):
            continue
        contamination = 1 - precision
        for i in range(len(max_contamination)):
            for j in range(len(min_completeness)):
                if recall > min_completeness[j] and contamination < max_contamination[i]:
                    genome_recovery[i][j] += 1
    return genome_recovery


def calc_dict(metrics, min_completeness, max_contamination):
    if not min_completeness:
        min_completeness = get_defaults()[0]
    if not max_contamination:
        max_contamination = get_defaults()[1]

    genome_recovery = calc_table(metrics, min_completeness, max_contamination)
    counts_dict = collections.OrderedDict()
    for i, cont in zip(range(len(max_contamination)), max_contamination):
        for j, compl in zip(range(len(min_completeness)), min_completeness):
            counts_dict['>{}compl<{}cont'.format(compl, cont)] = genome_recovery[i][j]
    return counts_dict


def print_table(genome_recovery, label, min_completeness, max_contamination, stream=sys.stdout):
    if not label:
        label = ""
    if not min_completeness:
        min_completeness = get_defaults()[0]
    if not max_contamination:
        max_contamination = get_defaults()[1]

    completeness = "".join(">{}% complete\t".format(str(int(x * 100))) for x in min_completeness).rstrip("\t")
    contamination = ["<{}% contamination".format(str(int(x * 100))) for x in max_contamination]

    stream.write("%s\t%s\n" % (label, completeness))

    for y_label, num_genomes in zip(contamination, genome_recovery):
        contamination_values = "".join("{}\t".format(str(values)) for values in num_genomes).rstrip("\t")
        stream.write("%s\t%s\n" % (y_label, contamination_values))


def main():
    parser = argparse.ArgumentParser(description="Calculate number of genome bins recovered with more than the specified thresholds of completeness and contamination. Default: >50%, >70%, >90% completeness vs. <10%, <5% contamination",
                                     parents=[argparse_parents.PARSER_MULTI])
    parser.add_argument('-x', '--min_completeness', help=argparse_parents.HELP_THRESHOLDS_COMPLETENESS, required=False)
    parser.add_argument('-y', '--max_contamination', help=argparse_parents.HELP_THRESHOLDS_CONTAMINATION, required=False)
    args = parser.parse_args()
    if not args.file and sys.stdin.isatty():
        parser.print_help()
        parser.exit(1)
    metrics = load_data.load_tsv_table(sys.stdin if not sys.stdin.isatty() else args.file)

    min_completeness = None
    max_contamination = None
    if args.min_completeness:
        min_completeness = [int(x.strip())/100.0 for x in args.min_completeness.split(',')]
    if args.max_contamination:
        max_contamination = [int(x.strip())/100.0 for x in args.max_contamination.split(',')]

    if args.filter:
        metrics = filter_tail.filter_tail(metrics, args.filter)
    results = calc_table(metrics, min_completeness, max_contamination)
    print_table(results, args.label, min_completeness, max_contamination)


if __name__ == "__main__":
    main()
